#!/usr/bin/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test the DelphesApp WorkflowModule."""

from __future__ import print_function
from __future__ import absolute_import
import unittest
import os
import os.path
import shutil
import tempfile
from mock import patch, MagicMock as Mock, mock_open

from parameterized import parameterized, param

from DIRAC import gLogger, S_OK, S_ERROR
from ILCDIRAC.Workflow.Modules.DelphesAppModule import DelphesAppModule
from Tests.Utilities.GeneralUtils import assertDiracSucceeds, assertDiracFails

__RCSID__ = "$Id$"


MODULE_NAME = 'ILCDIRAC.Workflow.Modules.DelphesAppModule'
MODULEBASE_NAME = 'ILCDIRAC.Workflow.Modules.ModuleBase'
PROXYINFO_NAME = 'DIRAC.Core.Security.ProxyInfo'
# pylint: disable=too-many-public-methods, protected-access

gLogger.setLevel("ERROR")
gLogger.showHeaders(True)


def cleanup(tempdir):
  """Remove files after run."""
  try:
    shutil.rmtree(tempdir)
  except OSError:
    pass


@patch("%s.getProxyInfoAsString" % MODULEBASE_NAME, new=Mock(return_value=S_OK()))
@patch("%s.getProxyInfoAsString" % PROXYINFO_NAME, new=Mock(return_value=S_OK()))
class TestDelphesAppModule(unittest.TestCase):
  """test DelphesAppModule."""

  def assertIn(self, *args, **kwargs):
    """make this existing to placate pylint."""
    return super(TestDelphesAppModule, self).assertIn(*args, **kwargs)

  @patch("%s.getProxyInfoAsString" % MODULEBASE_NAME, new=Mock(return_value=S_OK()))
  @patch("%s.getProxyInfoAsString" % PROXYINFO_NAME, new=Mock(return_value=S_OK()))
  def setUp(self):
    self.delphes = DelphesAppModule()
    self.curdir = os.getcwd()
    self.tempdir = tempfile.mkdtemp("", dir="./")
    os.chdir(self.tempdir)
    self.delphes.ops = Mock()

  def tearDown(self):
    os.chdir(self.curdir)
    cleanup(self.tempdir)

#---------------------------------------------------------------------------

class TestDelphesAppModuleRunit(TestDelphesAppModule):
  """test DelphesApp runtIt."""

  def setUp(self):
    super(TestDelphesAppModuleRunit, self).setUp()
    self.logFileName = "localEnv.log"
    self.executableName = "DelphesPythia8_EDM4HEP"
    self.detectorCard = "delphes_card_IDEA.tcl"
    self.outputCard = "edm4hep_output_config.tcl"
    self.pythia8CardContent = "Random:setSeed = on\nMain:numberOfEvents = 3000         ! number of events to generate"
    with open(self.logFileName, "w") as logF:
      logF.write("logged the logging logs")

#-------------------------- RUNIT

  def test_DelphesApp_runIt_failure_NoPlatform(self):
    """DelphesApp.runit failure with platform ........................................................."""
    self.delphes.applicationLog = self.logFileName
    self.delphes.executableName = self.executableName
    self.delphes.detectorCard = self.detectorCard
    self.delphes.outputCard = self.outputCard
    res = self.delphes.runIt()
    self.assertIn("No ILC platform selected", res['Message'])

  def test_DelphesApp_runIt_failure_NoLogFile(self):
    """delphes.runit failure with logfile................................................................"""
    self.delphes.platform = "Windows"
    self.delphes.executableName = self.executableName
    self.delphes.detectorCard = self.detectorCard
    self.delphes.outputCard = self.outputCard
    res = self.delphes.runIt()
    self.assertIn("No Log file provided", res['Message'])

  def test_DelphesApp_runIt_failure_NoExecutable(self):
    """delphes.runit failure with executable name................................................................"""
    self.delphes.platform = "Windows"
    self.delphes.applicationLog = self.logFileName
    self.delphes.detectorCard = self.detectorCard
    self.delphes.outputCard = self.outputCard
    res = self.delphes.runIt()
    self.assertIn("No executable name provided", res['Message'])

  def test_DelphesApp_runIt_failure_NodetectorCard(self):
    """delphes.runit failure with detectorCard name................................................................"""
    self.delphes.platform = "Windows"
    self.delphes.applicationLog = self.logFileName
    self.delphes.executableName = self.executableName
    self.delphes.outputCard = self.outputCard
    res = self.delphes.runIt()
    self.assertIn("No detectorCard name provided", res['Message'])

  def test_DelphesApp_runIt_failure_NooutputCard(self):
    """delphes.runit failure with outputCard name................................................................"""
    self.delphes.platform = "Windows"
    self.delphes.applicationLog = self.logFileName
    self.delphes.executableName = self.executableName
    self.delphes.detectorCard = self.detectorCard
    res = self.delphes.runIt()
    self.assertIn("No outputCard name provided", res['Message'])

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_ERROR("missing setup.sh")))
  def test_DelphesApp_runIt_failure_env(self):
    """delphes.runit failed to get env................................................................"""
    self.delphes.platform = "Windows"
    self.delphes.applicationLog = self.logFileName
    self.delphes.executableName = self.executableName
    self.delphes.detectorCard = self.detectorCard
    self.delphes.outputCard = self.outputCard
    res = self.delphes.runIt()
    self.assertEqual(res['Message'], "missing setup.sh")

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value={"OK": False, "Message": ""}))
  def test_DelphesApp_runIt_failure_wrong_env_res(self):
    """delphes.runit failed to obtain the environment script................................................................"""
    self.delphes.platform = "Windows"
    self.delphes.applicationLog = self.logFileName
    self.delphes.executableName = self.executableName
    res = self.delphes.runIt()
    assertDiracFails(res, self)

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("setup.sh")))
  def test_DelphesApp_runIt_success_workflowStatusOK(self):
    """delphes.runit success while having workflowStatus[OK] set as False................................................................"""
    self.delphes.platform = "Windows"
    self.delphes.applicationLog = self.logFileName
    self.delphes.executableName = self.executableName
    self.delphes.detectorCard = self.detectorCard
    self.delphes.outputCard = self.outputCard
    self.delphes.pythia8CardContent = self.pythia8CardContent
    self.delphes.workflowStatus['OK'] = False
    #NB: workflowStatus['OK'] = False will cause RunIt() to be interrupted earlier
    res = self.delphes.runIt()
    assertDiracSucceeds(res, self)

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("setup.sh")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  def test_DelphesApp_runIt_failure_pythia8Cardnotfound(self):
    """DelphesApp.runit failure with Pythia8 card not found..........................................."""
    self.delphes.platform = "Windows"
    self.delphes.applicationLog = self.logFileName
    self.delphes.executableName = self.executableName
    self.delphes.ignoreapperrors = False
    self.delphes.detectorCard = self.detectorCard
    self.delphes.outputCard = self.outputCard
    self.delphes.pythia8CardContent = self.pythia8CardContent
    # side effect for the os.path.exists in DelphesAppModule:
    # scriptName, "lib", self.applicationLog, self.applicationLog, self.pythia8Card
    with patch("os.path.exists", new=Mock(side_effect=[False, False, True, False, False])):
      res = self.delphes.runIt()
    self.assertIn("Pythia card with unusual content: neither `Beams:LHEF` nor `Beams:eCM` fields present. Please check the content of your card.", res['Message'])

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("setup.sh")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  def test_DelphesApp_runIt_success_applog(self):
    """DelphesApp.runit success with applog..........................................."""
    self.delphes.platform = "Windows"
    self.delphes.applicationLog = self.logFileName
    self.delphes.executableName = "DelphesROOT_EDM4HEP"
    self.delphes.ignoreapperrors = False
    self.delphes.detectorCard = self.detectorCard
    self.delphes.outputCard = self.outputCard
    self.delphes.pythia8CardContent = self.pythia8CardContent
    with open("p8_ee_ggqq_ecm91.cmd", "w") as scr:
      scr.write("Beams:LHEF\nMain:numberOfEvents")
    # side effect for the os.path.exists in DelphesAppModule:
    # scriptName, "lib", self.applicationLog, self.applicationLog, (self.pythia8Card)
    with patch("os.path.exists", new=Mock(side_effect=[False, False, False, True, True])):
      res = self.delphes.runIt()
    assertDiracSucceeds(res, self)

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("setup.sh")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  def test_DelphesApp_runIt_success_ignoreapperrors(self):
    """DelphesApp.runit success with ignoreappererrors..........................................."""
    self.delphes.platform = "Windows"
    self.delphes.applicationLog = self.logFileName
    self.delphes.executableName = "DelphesROOT_EDM4HEP"
    self.delphes.ignoreapperrors = True
    self.delphes.detectorCard = self.detectorCard
    self.delphes.outputCard = self.outputCard
    self.delphes.pythia8CardContent = self.pythia8CardContent
    with open("p8_ee_ggqq_ecm91.cmd", "w") as scr:
      scr.write("content")
    # side effect for the os.path.exists in DelphesAppModule:
    # scriptName, "lib", self.applicationLog, self.applicationLog, (self.pythia8Card)
    with patch("os.path.exists", new=Mock(side_effect=[False, False, False, False, True])):
      res = self.delphes.runIt()
    assertDiracSucceeds(res, self)

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("setup.sh")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  def test_DelphesApp_runIt_success_solveIFpaths(self):
    """DelphesApp.runit success with solveIFpaths..........................................."""
    self.delphes.platform = "Windows"
    self.delphes.applicationLog = self.logFileName
    self.delphes.executableName = "DelphesROOT_EDM4HEP"
    self.delphes.ignoreapperrors = True
    self.delphes.detectorCard = self.detectorCard
    self.delphes.outputCard = self.outputCard
    self.delphes.InputFile = 'some simulation'
    with open("p8_ee_ggqq_ecm91.cmd", "w") as scr:
      scr.write("content")
    # side effect for the os.path.exists in DelphesAppModule:
    # scriptName, "lib", self.applicationLog, self.applicationLog, (self.pythia8Card)
    with patch("os.path.exists", new=Mock(side_effect=[False, False, False, False, True])), \
      patch("%s.resolveIFpaths" % MODULE_NAME, new=Mock(return_value=S_OK('some simulation'))):
      res = self.delphes.runIt()
    assertDiracSucceeds(res, self)

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("setup.sh")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  def test_DelphesApp_runIt_success_solveIFpaths_notfound(self):
    """DelphesApp.runit fails with solveIFpaths not finding input files..........................................."""
    self.delphes.platform = "Windows"
    self.delphes.applicationLog = self.logFileName
    self.delphes.executableName = "DelphesROOT_EDM4HEP"
    self.delphes.ignoreapperrors = True
    self.delphes.detectorCard = self.detectorCard
    self.delphes.outputCard = self.outputCard
    self.delphes.pythia8CardContent = self.pythia8CardContent
    self.delphes.InputFile = 'some simulation'
    with patch("%s.resolveIFpaths" % MODULE_NAME, new=Mock(return_value=S_ERROR('resolveIFPath: Input file(s) not found locally'))):
      res = self.delphes.runIt()
    self.assertEqual(res['Message'], "resolveIFPath: Input file(s) not found locally")

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("setup.sh")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  def test_DelphesApp_runIt_failure_LogFile(self):
    """DelphesApp.runit failure with logfile......................................................"""
    self.delphes.platform = "Windows"
    self.delphes.applicationLog = self.logFileName
    self.delphes.executableName = "DelphesSTDHEP_EDM4HEP"
    self.delphes.ignoreapperrors = False
    self.delphes.detectorCard = self.detectorCard
    self.delphes.outputCard = self.outputCard
    self.delphes.pythia8CardContent = self.pythia8CardContent
    with open("p8_ee_ggqq_ecm91.cmd", "w") as scr:
      scr.write("content")
    # side effect for the os.path.exists in DelphesAppModule:
    # scriptName, "lib", self.applicationLog, self.applicationLog, (self.pythia8Card)
    with patch("os.path.exists", new=Mock(side_effect=[False, False, False, False, True])):
      res = self.delphes.runIt()
    print(res)
    self.assertIn("did not produce the expected log", res['Message'])

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("setup.sh")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  def test_DelphesApp_runIt_success_LogAndScriptPresentEditingThePythiaFile(self):
    """DelphesApp.runit success log and script exist, also editing the number of events in the Pythia file..................................................."""
    self.delphes.platform = "Windows"
    self.delphes.applicationLog = self.logFileName
    self.delphes.ignoreapperrors = True
    self.delphes.executableName = self.executableName
    self.delphes.detectorCard = self.detectorCard
    self.delphes.outputCard = self.outputCard
    self.delphes.pythia8CardContent = "Beams:LHEF\nMain:numberOfEvents"
    with open("DelphesApp__Run_.sh", "w") as scr:
      scr.write("content")
    with open(self.logFileName, "w") as scr:
      scr.write("content")
    # side effect for the os.path.exists in DelphesAppModule:
    # scriptName, "lib", self.applicationLog, self.pythia8Card, self.applicationLog
    with patch("os.path.exists", new=Mock(side_effect=[True, True, False, True, False])):    
      res = self.delphes.runIt()
    assertDiracSucceeds(res, self)

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("setup.sh")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  def test_DelphesApp_runIt_success_LogAndScriptPresentNotfindingthelineonThePythiaFile(self):
    """DelphesApp.runit success log and script exist, not succeding in editing the number of events in the Pythia file..................................................."""
    self.delphes.platform = "Windows"
    self.delphes.applicationLog = self.logFileName
    self.delphes.ignoreapperrors = True
    self.delphes.executableName = self.executableName
    self.delphes.detectorCard = self.detectorCard
    self.delphes.outputCard = self.outputCard
    self.delphes.pythia8CardContent = "Beams:LHEF\nMain:numberOfEvents"
    with open("DelphesApp__Run_.sh", "w") as scr:
      scr.write("content")
    with open(self.logFileName, "w") as scr:
      scr.write("content")
    # side effect for the os.path.exists in DelphesAppModule:
    # scriptName, "lib", self.applicationLog, self.pythia8Card, self.applicationLog
    with patch("os.path.exists", new=Mock(side_effect=[True, True, False, True, True])):    
      res = self.delphes.runIt()
    assertDiracSucceeds(res, self)

#--------------------------------ASI

class TestDelphesAppModuleASI(TestDelphesAppModule):
  """delphes.ApplicationSpecificInputs."""

  @patch.dict(os.environ, {"JOBID": "12345"})
  def test_DelphesApp_ASI_NoVariables(self):
    """delphes.applicationSpecificInputs: checks that no variables have been set after this call......"""
    gLogger.setLevel("ERROR")
    self.delphes.workflow_commons = dict()
    self.delphes.applicationSpecificInputs()
    self.assertFalse(self.delphes.jobReport or self.delphes.productionID)

  @patch.dict(os.environ, {"JOBID": "12345"})
  def test_DelphesAPP_ASI_success_Randomseed(self):
    """DelphesApp.applicationSpecificInputs success with a random seed............................................."""
    self.delphes.workflow_commons = {"IS_PROD": True,
                                     "PRODUCTION_ID": '2345',
                                     "JOB_ID": '12345'}
    self.delphes.OutputFile = 'outputfile.root'
    self.delphes.randomSeed = 10
    res = self.delphes.applicationSpecificInputs()
    assertDiracSucceeds(res, self)

  @patch.dict(os.environ, {"JOBID": "12345"})
  def test_DelphesAPP_ASI_success_workflowstartfrom_Inputdata(self):
    """DelphesApp.applicationSpecificInputs success with a workflowstartfrom............................................."""
    self.delphes.InputData = "inputdata"
    self.delphes.WorkflowStartFrom = True     
    res = self.delphes.applicationSpecificInputs()
    assertDiracSucceeds(res, self)

#-----------------------------------------------------------------------------------------

def runTests():
  """Runs our tests."""
  suite = unittest.defaultTestLoader.loadTestsFromTestCase(TestDelphesAppModule)
  suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(TestDelphesAppModuleRunit))
  suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(TestDelphesAppModuleASI))
  testResult = unittest.TextTestRunner(verbosity=2).run(suite)
  print(testResult)


#if __name__ == '__main__':
runTests()



#!/usr/local/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test SLCIOSplit module."""

from __future__ import absolute_import
import unittest
from mock import patch, MagicMock as Mock

from DIRAC import S_OK, S_ERROR
from ILCDIRAC.Interfaces.API.NewInterface.Applications import SLCIOSplit
from Tests.Utilities.GeneralUtils import assertEqualsImproved, assertDiracFailsWith, \
    assertDiracSucceeds, assertInImproved

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Interfaces.API.NewInterface.Applications.SLCIOSplit'

# pylint: disable=protected-access


class SLCIOSplitTestCase(unittest.TestCase):
  """Base class for the SLCIOSplit test cases."""

  def setUp(self):
    """set up the objects."""
    self.ssp = SLCIOSplit({})

  def test_setnumberevtsperfile(self):
    self.ssp.setNumberOfEventsPerFile(987124)
    assertEqualsImproved(self.ssp.numberOfEventsPerFile, 987124, self)
    self.assertFalse(self.ssp._errorDict)

  def test_setnumberevtsperfile_fails(self):
    self.ssp.setNumberOfEventsPerFile({'asf': True})
    assertInImproved('_checkArgs', self.ssp._errorDict, self)

  def test_userjobmodules(self):
    module_mock = Mock()
    assertDiracSucceeds(self.ssp._userjobmodules(module_mock), self)

  def test_prodjobmodules(self):
    module_mock = Mock()
    assertDiracSucceeds(self.ssp._prodjobmodules(module_mock), self)

  def test_userjobmodules_fails(self):
    with patch('%s._setUserJobFinalization' % MODULE_NAME, new=Mock(return_value=S_OK('something'))),\
        patch('%s._setApplicationModuleAndParameters' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_test_err'))):
      assertDiracFailsWith(self.ssp._userjobmodules(None),
                            'userjobmodules failed', self)

  def test_prodjobmodules_fails(self):
    with patch('%s._setApplicationModuleAndParameters' % MODULE_NAME, new=Mock(return_value=S_OK('something'))), \
        patch('%s._setOutputComputeDataList' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_other_test_err'))):
      assertDiracFailsWith(self.ssp._prodjobmodules(None),
                            'prodjobmodules failed', self)

  def test_checkworkflowconsistency(self):
    self.ssp._checkWorkflowConsistency()
    # pylint: disable=redundant-unittest-assert
    self.assertTrue(True)

  def test_resolvelinkedstepparams(self):
    instance_mock = Mock()
    step_mock = Mock()
    step_mock.getType.return_value = 'abc'
    self.ssp._inputappstep = None
    self.ssp._jobsteps = ['', '', step_mock]
    self.ssp._linkedidx = 2
    assertDiracSucceeds(self.ssp._resolveLinkedStepParameters(instance_mock), self)
    instance_mock.setLink.assert_called_once_with('InputFile', 'abc', 'OutputFile')

  def test_resolvelinkedstepparams_nothing_happens(self):
    instance_mock = Mock()
    self.ssp._inputappstep = None
    self.ssp._jobsteps = None
    self.ssp._linkedidx = ['abc']
    assertDiracSucceeds(self.ssp._resolveLinkedStepParameters(instance_mock), self)
    self.assertFalse(instance_mock.setLink.called)

  def test_checkconsistency(self):
    job_mock = Mock()
    job_mock.datatype = 'Mock_type_data'
    job_mock.detector = 'testdetector123'
    self.ssp._job = job_mock
    self.ssp._jobtype = 'notUser'
    self.ssp.OutputFile = None
    assertDiracSucceeds(self.ssp._checkConsistency(), self)
    assertInImproved({'outputFile': '@{OutputFile}', 'outputPath': '@{OutputPath}',
                        'outputDataSE': '@{OutputSE}'}, self.ssp._listofoutput, self)
    assertEqualsImproved((self.ssp.datatype, self.ssp.detectortype),
                          ('Mock_type_data', 'testdetector123'), self)

  def test_checkconsistency_userjob(self):
    self.ssp._jobtype = 'User'
    self.ssp.OutputFile = None
    assertDiracSucceeds(self.ssp._checkConsistency(), self)
    self.assertFalse(self.ssp.outputFile)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test the dirac-fcc-make-productions script."""

from __future__ import absolute_import
import os
import importlib
import six.moves.configparser
from collections import defaultdict

import pytest
from mock import MagicMock as Mock, patch

from DIRAC import S_OK, S_ERROR

from ILCDIRAC.ILCTransformationSystem.Utilities.Utilities import Task
from DIRAC.ConfigurationSystem.Client.Helpers.Operations import Operations

from parameterized import parameterized

# pylint: disable=protected-access, invalid-name, missing-docstring, redefined-outer-name
THE_SCRIPT = "ILCDIRAC.ILCTransformationSystem.scripts.dirac_fcc_make_productions"
theScript = importlib.import_module(THE_SCRIPT)
SCP = "ILCDIRAC.ILCTransformationSystem.scripts.dirac_fcc_make_productions.six.moves.configparser.SafeConfigParser"
__RCSID__ = "$Id$"

CONFIG_DICT = {}

def test_environ():
  """Ensure we are not running optimized python.

  Some assertion in the pytest code are otherwise ignored.
  """
  assert "PYTHONOPTIMIZE" not in os.environ


def configDict():
  """Return dictionary for dirac configuration system."""
  return {'prodGroup': "myProdGroup",
          'detectorModel': 'myDetectorModel',
          'softwareVersion': 'mySoftwareVersion',
          'configVersion': 'key4hep-devel-2',
          'configPackage': 'fccConfig',
          'machine': 'ee',
          'generatorApplication': 'KKMC',
          'generatorSteeringFile': 'somesteeringfile',
          'campaign': 'winter2023',
          'datatype': 'lhe',
          'processes': 'qqbar, WW_incl',
          'energies': '100, 200',
          'processingAfterGen': 'delphesapp',
          'eventsPerJobs': '1000, 2000',
          'productionloglevel': 'DEBUGLEVEL3',
          'outputSE': 'CERN-DST-EOS',
          'finalOutputSE': 'CERN-SRM',
          'additionalName': 'waitForIt',
          'prodIDs': '123, 456',
          'eventsInSplitFiles': '5000, 6000',
          'ProdTypes': 'Gen',
          'MoveTypes': '',
          'MoveStatus': 'Active',
          'MoveGroupSize': '11',
          'overlayEvents': '',
          'overlayEventType': '',
          'cliReco': '--Config.Tracking=Tracked',
          'numberOfTasks': '1, 2',
          'ignoreMetadata': '',
          'taskNames': 'taskA, taskB',
          }


@pytest.fixture
def opsMock():
  """Return fixture for Operations."""
  def mockOpsConfig(*args, **kwargs):  # pylint: disable=unused-argument
    """Mock the operations getValue calls."""
    opsDict = {'DefaultDetectorModel': 'detModel',
               'DefaultConfigVersion': 'Config',
               'DefaultConfigPackage': 'Click',
               'DefaultSoftwareVersion': 'Software',
               'FailOverSE': 'FAIL=SRM',
               'DefaultDelphesAppVersion': '1.9.5',
               'BasePath': '',
              }
    for opName, value in opsDict.items():
      if args[0].endswith(opName):
        return value
    assert args[0] == opsDict
    return None 
  theOps = Mock(name='OpsMock')
  theOps.getValue = Mock(side_effect=mockOpsConfig)
  return theOps


@pytest.fixture
def theChain(opsMock):
  """Return production chain fixture."""
  params = Mock()
  params.additionalName = ''
  params.dryRun = True
  with patch("ILCDIRAC.ILCTransformationSystem.scripts.dirac_fcc_make_productions.FCCDetProdChain.loadParameters",
             new=Mock()), \
       patch("DIRAC.ConfigurationSystem.Client.Helpers.Operations",
             new=Mock(return_value=opsMock)):
    chain = theScript.FCCDetProdChain(params)

  return chain


@pytest.fixture
def aTask():
  """Return aTask with minimal content."""
  task = Task({'Energy': 300.0}, {}, 333)
  return task


@pytest.fixture
def cpMock():
  """Return a Mock for the ConfigParser."""
  theCPMock = Mock()
  theCPMock.thisConfigDict = dict(configDict())

  def hasMock(*args, **kwargs):  # pylint: disable=unused-argument
    """Mock the configparser.has_option function."""
    return theCPMock.thisConfigDict.get(args[1])

  def mockConfig(*args, **kwargs):  # pylint: disable=unused-argument
    """Mock the configparser object."""
    assert args[0] == theScript.PP
    return theCPMock.thisConfigDict[args[1]]

  theCPMock.read = Mock()
  theCPMock.get = mockConfig
  theCPMock.has_option = hasMock
  return theCPMock


def test_meta(theChain):
  """Test meta data."""
  ret = theChain.meta(123, 'process', 555.5)
  assert {'ProdID': '123',
          'EvtType': 'process',
          'Energy': '555.5',
          'Machine': 'ee'} == ret
  theChain.ignoreMetadata = ['EvtType']
  ret = theChain.meta(123, 'process', 555.5)
  assert {'ProdID': '123',
          'Energy': '555.5',
          'Machine': 'ee'} == ret


def test_loadParameters(theChain, cpMock):
  """Test load parameters."""
  parameter = Mock()
  parameter.prodConfigFilename = None
  parameter.dumpConfigFile = None
  theChain.loadParameters(parameter)
  c = theChain

  parameter.prodConfigFilename = 'filename'

  with patch(SCP, new=Mock(return_value=cpMock)):
    c.loadParameters(parameter)
  assert c.prodGroup == "myProdGroup"
  assert c.detectorModel == "myDetectorModel"
  assert c.prodIDs == [123, 456]
  assert c.energies == [100, 200]
  assert c.eventsPerJobs == [1000, 2000]
  assert c.eventsInSplitFiles == [5000, 6000]

  assert c.moveStatus == 'Active'
  assert c.moveGroupSize == '11'

  cpMock.thisConfigDict['prodIDs'] = "123, 456, 789"
  with patch(SCP, new=Mock(return_value=cpMock)), \
       pytest.raises(AttributeError, match="Lengths of Processes"):
    c.loadParameters(parameter)

  cpMock.thisConfigDict['prodIDs'] = ''
  cpMock.has_option = Mock()
  cpMock.has_option.return_value = False
  with patch(SCP, new=Mock(return_value=cpMock)):
    c.loadParameters(parameter)
  assert c.prodIDs == [1, 1]
  assert c.cliRecoOption == '--Config.Tracking=Tracked'

  cpMock.thisConfigDict['eventsInSplitFiles'] = "1000"
  c._flags._spl = True
  with patch(SCP, new=Mock(return_value=cpMock)), \
       pytest.raises(AttributeError, match="Length of eventsInSplitFiles"):
    c.loadParameters(parameter)

  parameter.prodConfigFilename = None
  parameter.dumpConfigFile = True
  with patch(SCP, new=Mock(return_value=cpMock)), \
    pytest.raises(RuntimeError, match="^$"):
    c.loadParameters(parameter)

  parameter.prodConfigFilename = 'filename'
  parameter.dumpConfigFile = False
  cpMock.thisConfigDict['MoveStatus'] = 'Foo'
  with patch(SCP, new=Mock(return_value=cpMock)), \
       pytest.raises(AttributeError, match='MoveStatus can only be'):
    c.loadParameters(parameter)


def test_createMovingTransformation(theChain):
  """Test creating the moving productions."""
  theChain.outputSE = "Source"
  theChain.finalOutputSE = "Target"
  theChain.moveGroupSize = 13
  theChain._flags._rec = True
  theChain._flags._sim = True
  theChain._flags._moveDst = True
  theChain._flags._moveRec = False
  theChain._flags._moveSim = True
  theChain._flags._moves = True
  theChain._flags._dryRun = False
  with patch("DIRAC.TransformationSystem.Utilities.ReplicationTransformation.createDataTransformation") as moveMock:
    theChain.createMovingTransformation({'ProdID': 666}, 'MCReconstruction')
    parDict = dict(flavour='Moving',
                   targetSE='Target',
                   sourceSE='Source',
                   plugin='Broadcast',
                   metaKey='ProdID',
                   metaValue=666,
                   extraData={'Datatype': 'DST'},
                   tGroup='several',
                   groupSize=13,
                   enable=True,
                   )
    moveMock.assert_called_once_with(**parDict)

  with patch("DIRAC.TransformationSystem.Utilities.ReplicationTransformation.createDataTransformation") as moveMock:
    theChain.createMovingTransformation({'ProdID': 666}, 'MCSimulation')
    parDict = dict(flavour='Moving',
                   targetSE='Target',
                   sourceSE='Source',
                   plugin='BroadcastProcessed',
                   metaKey='ProdID',
                   metaValue=666,
                   extraData={'Datatype': 'SIM'},
                   tGroup='several',
                   groupSize=13,
                   enable=True,
                   )
    moveMock.assert_called_once_with(**parDict)

  theChain._flags._rec = True
  theChain._flags._moves = False
  theChain._flags._dryRun = False
  with patch("DIRAC.TransformationSystem.Utilities.ReplicationTransformation.createDataTransformation") as moveMock:
    theChain.createMovingTransformation({'ProdID': 666}, 'MCReconstruction')
    moveMock.assert_not_called()

  with pytest.raises(RuntimeError, match='ERROR creating Moving'):
    theChain.createMovingTransformation({'ProdID': 666}, "Split")

def test_setApplicationOptions(theChain, aTask):
  """Test setting the application options."""
  application = Mock()
  application.setSomeParameter = Mock()
  aTask.applicationOptions = {'foo': 'bar'}
  theChain.applicationOptions['AppName'] = {'SomeParameter': 'SomeValue', 'FE.foo': ['bar', 'baz'],
                                            'C_Repl': 'longValueWeDoNotwantToRepeat'}
  theChain._setApplicationOptions('AppName', application, aTask.applicationOptions)
  application.setSomeParameter.assert_called_once_with('SomeValue')
  application.setfoo.assert_called_once_with('bar')

  from ILCDIRAC.Interfaces.API.NewInterface.Applications.DelphesApp import DelphesApp
  application = DelphesApp()
  theChain.applicationOptions['AppName'] = {'SomeOtherParameter': 'SomeValue'}
  with pytest.raises(AttributeError, match='Cannot set'):
    theChain._setApplicationOptions('AppName', application)

def test_getProdInfoFromIDs(theChain):
  """Test getting theproduction information."""
  # successful
  theChain.prodIDs = [12345]
  trClientMock = Mock(name='trClient')
  trClientMock.getTransformation.return_value = S_OK({'EventsPerTask': 123})
  trMock = Mock(return_value=trClientMock)
  fcClientMock = Mock(name='fcClient')
  fcClientMock.findFilesByMetadata.return_value = S_OK(['/path/to/file'])
  fcClientMock.getDirectoryUserMetadata.return_value = S_OK({'EvtType': 'haha', 'Energy': 321})
  fcMock = Mock(return_value=fcClientMock)
  with patch('DIRAC.TransformationSystem.Client.TransformationClient.TransformationClient', new=trMock), \
       patch('DIRAC.Resources.Catalog.FileCatalogClient.FileCatalogClient', new=fcMock):
    theChain._getProdInfoFromIDs()
  assert theChain.eventsPerJobs == [123]
  assert theChain.processes == ['haha']
  assert theChain.energies == [321]

  # first exception
  theChain.prodIDs = []
  with pytest.raises(AttributeError, match='No prodIDs'):
    theChain._getProdInfoFromIDs()

  # second exception
  theChain.prodIDs = [12345]
  trClientMock = Mock(name='trClient')
  trClientMock.getTransformation.return_value = S_ERROR('No such prod')
  trMock = Mock(return_value=trClientMock)
  with patch('DIRAC.TransformationSystem.Client.TransformationClient.TransformationClient', new=trMock), \
       pytest.raises(AttributeError, match='No prodInfo found'):
    theChain._getProdInfoFromIDs()

  # third exception
  theChain.prodIDs = [12345]
  trClientMock = Mock(name='trClient')
  trClientMock.getTransformation.return_value = S_OK({'EventsPerTask': 123})
  trMock = Mock(return_value=trClientMock)
  fcClientMock = Mock(name='fcClient')
  fcClientMock.findFilesByMetadata.return_value = S_ERROR('No files found')
  fcMock = Mock(return_value=fcClientMock)
  with patch('DIRAC.TransformationSystem.Client.TransformationClient.TransformationClient', new=trMock), \
       patch('DIRAC.Resources.Catalog.FileCatalogClient.FileCatalogClient', new=fcMock), \
       pytest.raises(AttributeError, match='Could not find file'):
    theChain._getProdInfoFromIDs()

def test_createTransformations(theChain):
  """Test replcation transformation creation."""
  theChain.createMovingTransformation = Mock(name='MovingTrafo')
  taskDict = dict(MOVE_GEN=[{'move': 'gen'}],
                  MOVE_SPLIT=[{'move': 'split'}],
                  MOVE_SIM=[{'move': 'sim'}],
                  MOVE_REC=[{'move': 'rec'}],
                  MOVE_OVER=[{'move': 'over'}],
                )
  theChain.createTransformations(taskDict)
  theChain.createMovingTransformation.assert_any_call({'move': 'gen'}, 'MCGeneration')
  theChain.createMovingTransformation.assert_any_call({'move': 'split'}, 'MCGeneration')
  theChain.createMovingTransformation.assert_any_call({'move': 'sim'}, 'MCSimulation')
  theChain.createMovingTransformation.assert_any_call({'move': 'rec'}, 'MCReconstruction')
  theChain.createMovingTransformation.assert_any_call({'move': 'over'}, 'MCReconstruction_Overlay')

def test_addGenTask(theChain):
  """Test adding gen task."""
  taskDict = defaultdict(list)
  theChain.generatorApplication = 'delphesapp'
  theChain.addGenTask(taskDict, originalTask=Task({}, {}, 123))
  assert len(taskDict['GEN']) == 1

  taskDict = defaultdict(list)
  theChain.applicationOptions['delphesapp']['FE.additionalName'] = ['1', '2']
  theChain.applicationOptions['delphesapp']['steeringFile'] = ['steeringfile']
  theChain.addGenTask(taskDict, originalTask=Task({}, {}, 123))
  assert len(taskDict['GEN']) == 2
  assert taskDict['GEN'][0].taskName == '1'
  assert taskDict['GEN'][1].taskName == '2'

def test_createTaskDict_none(theChain):
  """Test createTaskDict function."""
  taskDict = theChain.createTaskDict(123456, 'ee_qq', 5000, 333, sinFile='file', nbTasks=222,
                                     eventsPerBaseFile=None, taskName='')
  for pType in ['GEN', 'SIM', 'REC', 'SPLIT']:
    assert not taskDict[pType]
  
def test_createTaskDict_gen(theChain):
  """Test createTaskDict function."""
  theChain._flags._gen = True
  taskDict = theChain.createTaskDict(123456, 'ee_qq', 5000, 333, sinFile='file', nbTasks=222,
                                     eventsPerBaseFile=None, taskName='')
  assert len(taskDict['GEN']) == 1
  assert taskDict['GEN'][0].genFile == 'file'
  assert taskDict['GEN'][0].nbTasks == 222
  assert taskDict['GEN'][0].meta == {'ProdID': '123456',
                                    'EvtType': 'ee_qq',
                                    'Energy': '5000',
                                    'Machine': 'ee',
                                    'NumberOfEvents': 333,
                                    }

@pytest.fixture
def theFlags():
  """Return the flags fixture."""
  return theScript.FCCDetProdChain.Flags()

def test_flags_init(theFlags):
  """Returns the flags constructor."""
  f = theFlags
  assert f._dryRun
  assert not f._gen
  assert not f._spl
  assert not f._sim
  assert not f._rec
  assert not f._over
  assert not f._moves
  assert not f._moveGen
  assert not f._moveSim
  assert not f._moveRec
  assert not f._moveDst

def test_flags_properties(theFlags):
  """Test the flags properties."""
  f = theFlags
  f._gen = True
  f._spl = True
  f._sim = True
  f._rec = False
  f._over = True
  assert f.dryRun
  assert f.gen
  assert f.spl
  assert f.sim
  assert not f.rec
  assert f.over
  f._dryRun = True
  f._moves = True
  f._moveRec = True
  assert f.move
  assert not f.moveGen
  assert not f.moveSim
  assert f.moveRec
  assert not f.moveDst
  f._dryRun = False
  f._moveGen = True
  f._moveSim = True
  f._moveRec = True
  f._moveDst = False
  assert f.move
  assert f.moveGen
  assert f.moveSim
  assert f.moveRec
  assert not f.moveDst

def test_flags_str(theFlags):
  """Test the flags string representation."""
  theFlags._gen = True
  theFlags._sim = True
  theFlags._rec = False
  theFlags._over = True
  flagStr = str(theFlags)
  assert flagStr == """

#Productions to create: Gen, Split, Sim, Rec, RecOver
ProdTypes = Gen, Sim, RecOver

move = False

#Datatypes to move: Gen, Sim, Rec, Dst
MoveTypes = \n"""

def test_loadFlags(theFlags):
  """Test loading the flags from config."""
  myConfig = six.moves.configparser.ConfigParser()
  myConfig.add_section(theScript.PRODUCTION_PARAMETERS)
  myConfig.set(theScript.PRODUCTION_PARAMETERS, 'ProdTypes', 'Gen, Sim,Rec')
  myConfig.set(theScript.PRODUCTION_PARAMETERS, 'move', 'False')
  myConfig.set(theScript.PRODUCTION_PARAMETERS, 'MoveTypes', 'gen, dst')
  theFlags.loadFlags(myConfig)
  f = theFlags
  assert f.gen
  assert f.sim
  assert f.rec
  assert not f.over
  assert f._moveGen
  assert not f._moveSim
  assert not f._moveRec
  assert f._moveDst
  myConfig.set(theScript.PRODUCTION_PARAMETERS, 'MoveTypes', 'gen, dst, badType')
  with pytest.raises(AttributeError, match='badType'):
    theFlags.loadFlags(myConfig)

@pytest.fixture
def theParams():
  """Return the Params fixture."""
  return theScript._Params()

def test_params_init(theParams):
  """Test the Params constructor."""
  assert theParams.prodConfigFilename is None
  assert not theParams.dumpConfigFile
  assert theParams.dryRun
  assert theParams.additionalName == ''

def test_params_setters(theParams):
  """Test the Params setters."""
  with patch("%s.os.path.exists" % THE_SCRIPT, new=Mock(return_value=True)):
    assert theParams.setProdConf('myconf')['OK']

  assert theParams.prodConfigFilename, 'myconf'
  assert theParams.setDumpConf('_')['OK']
  assert theParams.dumpConfigFile
  assert theParams.setEnable('_')['OK']
  assert not theParams.dryRun
  assert theParams.setAddName('addName')['OK']
  assert theParams.additionalName, 'addName'

# -------------------------------------------------------------------------

def test_basepath(theChain):
  theChain.machine = 'ee'
  theChain.campaign = 'winter2023'

  theChain._basepath = ''
  with patch("DIRAC.ConfigurationSystem.Client.Helpers.Operations.Operations.getValue", new=Mock(return_value="/fcc/ee/")):
    basepath = theChain.basepath
  assert basepath == "/fcc/ee/winter2023/"

  theChain._basepath = "/fcc/hh/"
  basepath = theChain.basepath
  assert basepath == "/fcc/hh/winter2023/"

def test_createGeneratorApplication(theChain, aTask):
    from ILCDIRAC.Interfaces.API.NewInterface.Applications import DelphesApp
    theChain.generatorApplication = 'delphesapp'
    aTask.datatype = 'sim'
    genApp = theChain.createGeneratorApplication(aTask)
    assert isinstance(genApp, DelphesApp)
#    assert genApp.datatype == aTask.datatype

@pytest.fixture
def pMockMod():
  """Return Module for ProductionJob."""
  pMockMod = Mock()
  pjMock = Mock(name="ProductionJob")
  pMockMod.return_value = pjMock
  pjMock.getMetadata.return_value = {}
  return pMockMod

def test_createGenProduction(theChain, aTask, pMockMod):
  """Test creating the generation production."""
  theChain._basepath = '/fcc/ee/'
  with patch("ILCDIRAC.Interfaces.API.NewInterface.ProductionJob.ProductionJob", new=pMockMod):
    aTask.parameterDict['process'] = 'qqbar'
    aTask.meta = {'Energy': '240',
                  'EvtType': 'ZH_incl',
                  'Machine': 'ee',
                  'NumberOfEvents': 1500,
                  'ProdID': '1'}
    aTask.eventsPerJob = 100
    retMeta = theChain.createGenerationProduction(aTask)
  assert retMeta == {}

def test_createGenProduction2(theChain, aTask, pMockMod):
  """Test creating the generation production, with a processing step after generation"""
  theChain._basepath = '/fcc/ee/'
  with patch("ILCDIRAC.Interfaces.API.NewInterface.ProductionJob.ProductionJob", new=pMockMod):
    aTask.parameterDict['process'] = 'qqbar'
    aTask.meta = {'Energy': '240',
                  'EvtType': 'ZH_incl',
                  'Machine': 'ee',
                  'NumberOfEvents': 1500,
                  'ProdID': '1'}
    aTask.eventsPerJob = 100
    theChain.processingAfterGen = 'delphesapp'
    retMeta = theChain.createGenerationProduction(aTask)
  assert retMeta == {}

# -------------------------------------------------------------------------

def test_createKKMCApplication(theChain, aTask, cpMock):
  """Test creating the kkmc application."""
  from ILCDIRAC.Interfaces.API.NewInterface.Applications import KKMC

  parameter = Mock(name="ParameterMock")
  parameter.dumpConfigFile = False
  with patch(SCP, new=Mock(return_value=cpMock)):
    theChain.loadParameters(parameter)

  aTask.meta = {'Energy': '240',
                'EvtType': 'ZH_incl',
                'Machine': 'ee',
                'NumberOfEvents': 1500,
                'ProdID': '1'}
  aTask.eventsPerJob = 100

  ret = theChain.createKKMCApplication(aTask)
  assert isinstance(ret, KKMC)
  assert ret.datatype == 'hepmc'
  assert ret.version == 'mySoftwareVersion'

def test_createBabayagaApplication(theChain, aTask, cpMock):
  """Test creating the babayaga application."""
  from ILCDIRAC.Interfaces.API.NewInterface.Applications import Babayaga

  parameter = Mock(name="ParameterMock")
  parameter.dumpConfigFile = False
  with patch(SCP, new=Mock(return_value=cpMock)):
    theChain.loadParameters(parameter)

  aTask.meta = {'Energy': '240',
                'EvtType': 'ZH_incl',
                'Machine': 'ee',
                'NumberOfEvents': 1500,
                'ProdID': '1'}
  aTask.eventsPerJob = 100

  ret = theChain.createBabayagaApplication(aTask)
  assert isinstance(ret, Babayaga)
  assert ret.datatype == 'lhef'
  assert ret.version == 'mySoftwareVersion'

def test_createBhlumiApplication(theChain, aTask, cpMock):
  """Test creating the bhlumi application."""
  from ILCDIRAC.Interfaces.API.NewInterface.Applications import Bhlumi

  parameter = Mock(name="ParameterMock")
  parameter.dumpConfigFile = False
  with patch(SCP, new=Mock(return_value=cpMock)):
    theChain.loadParameters(parameter)

  aTask.meta = {'Energy': '240',
                'EvtType': 'ZH_incl',
                'Machine': 'ee',
                'NumberOfEvents': 1500,
                'ProdID': '1'}
  aTask.eventsPerJob = 100

  ret = theChain.createBhlumiApplication(aTask)
  assert isinstance(ret, Bhlumi)
  assert ret.datatype == 'lhef'
  assert ret.version == 'mySoftwareVersion'

def test_createDelphesApplication(theChain, aTask, cpMock):
  """Test creating the delphes application."""
  from ILCDIRAC.Interfaces.API.NewInterface.Applications import DelphesApp

  parameter = Mock(name="ParameterMock")
  parameter.dumpConfigFile = False
  with patch(SCP, new=Mock(return_value=cpMock)):
    theChain.loadParameters(parameter)

  aTask.meta = {'Energy': '240',
                'EvtType': 'ZH_incl',
                'Machine': 'ee',
                'NumberOfEvents': 1500,
                'ProdID': '1'}
  aTask.genFile = 'p8_ee_ZZ_ecm240.cmd'
  aTask.eventsPerJob = 100

  ret = theChain.createDelphesApplication(aTask)
  assert isinstance(ret, DelphesApp)
  assert ret.datatype == 'delphes'
  assert ret.version == 'mySoftwareVersion'

def test_createGaudiApplication(theChain, aTask, cpMock):
  """Test creating the delphes application."""
  from ILCDIRAC.Interfaces.API.NewInterface.Applications import GaudiApp

  parameter = Mock(name="ParameterMock")
  parameter.dumpConfigFile = False
  with patch(SCP, new=Mock(return_value=cpMock)):
    theChain.loadParameters(parameter)
  aTask.meta = {'Energy': '240',
                'EvtType': 'ZH_incl',
                'Machine': 'ee',
                'NumberOfEvents': 1500,
                'ProdID': '1'}
  aTask.applicationOptions = {'GaudiWorkFlow': 'fastsim'}
  aTask.eventsPerJob = 100
  aTask.genFile = 'p8_ee_ZZ_ecm240.cmd'

  ret = theChain.createGaudiApplication(aTask)
  assert isinstance(ret, GaudiApp)
  assert ret.datatype == 'delphes'
  assert ret.version == 'mySoftwareVersion'

def test_createGaudiApplication_2(theChain, aTask, cpMock):
  """Test creating the delphes application."""
  from ILCDIRAC.Interfaces.API.NewInterface.Applications import GaudiApp

  parameter = Mock(name="ParameterMock")
  parameter.dumpConfigFile = False
  with patch(SCP, new=Mock(return_value=cpMock)):
    theChain.loadParameters(parameter)

  aTask.meta = {'Energy': '240',
                'EvtType': 'ZH_incl',
                'Machine': 'ee',
                'NumberOfEvents': 1500,
                'ProdID': '1'}
  aTask.applicationOptions = {'GaudiWorkFlow': 'fullsim'}
  aTask.eventsPerJob = 100

  ret = theChain.createGaudiApplication(aTask)
  assert not ret.datatype
  assert isinstance(ret, GaudiApp)
  assert ret.version == 'mySoftwareVersion'

def test_createWhizard2Application(theChain, aTask, cpMock):
  """Test creating the delphes application."""
  from ILCDIRAC.Interfaces.API.NewInterface.Applications import Whizard2

  parameter = Mock(name="ParameterMock")
  parameter.dumpConfigFile = False
  with patch(SCP, new=Mock(return_value=cpMock)):
    theChain.loadParameters(parameter)

  aTask.meta = {'Energy': '240',
                'EvtType': 'ZH_incl',
                'Machine': 'ee',
                'NumberOfEvents': 1500,
                'ProdID': '1'}
  aTask.genFile = 'ee_ZHH_1500gev_polp80.sin'
  aTask.eventsPerJob = 100

  ret = theChain.createWhizard2Application(aTask)
  assert isinstance(ret, Whizard2)
  assert ret.datatype == "stdhep"
  assert ret.version == 'mySoftwareVersion'


#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Delphes Application to run applications based on Delphes.

.. versionadded:: v34r0

Usage:

>>> delphes = DelphesApp()
>>> delphes.setVersion('key4hep-latest')
>>> delphes.setExecutable("DelphesPythia8_EDM4HEP")
>>> delphes.setDetectorCard('card_IDEA.tcl')
>>> delphes.setOutputCard('edm4hep_IDEA.tcl')
>>> delphes.setPythia8Card('p8_ee_ggqq_ecm91.cmd')
>>> delphes.setRandomSeed(12340)
>>> delphes.setEnergy(91.2)
>>> delphes.setNumberOfEvents(100)
>>> delphes.setOutputFile('output.root')

"""
from __future__ import absolute_import
import types
import os

from ILCDIRAC.Interfaces.API.NewInterface.LCApplication import LCApplication
from ILCDIRAC.Core.Utilities.InstalledFiles import Exists
from DIRAC import S_OK, S_ERROR, gLogger
from DIRAC.Core.Workflow.Parameter import Parameter
from DIRAC.ConfigurationSystem.Client.Helpers.Operations import Operations
import six
from ILCDIRAC.Core.Utilities.PrepareOptionFiles import preparePythia8Card, PYTHIA_LHE_INPUT_CMD

LOG = gLogger.getSubLogger(__name__)
__RCSID__ = "$Id$"


class DelphesApp(LCApplication):
  """DelphesApp Application Class."""

  def __init__(self, paramdict=None):
    self.randomSeed = -1
    self.executableName = ''
    self.detectorCard = ''
    self.outputCard = ''
    self.pythia8CardContent = ''
    super(DelphesApp, self).__init__(paramdict)
    # Those 5 need to come after default constructor
    self._modulename = 'DelphesAppModule'
    self._moduledescription = 'Module to run DelphesApp'
    self.appname = 'delphesapp'
    self._ops = Operations()
    self._extension = 'root'

  def setRandomSeed(self, randomSeed):
    """Define random seed to use. Default is the jobID.

    :param int randomSeed: Seed to use during simulation.
    """
    self._checkArgs({'randomSeed': int})
    self.randomSeed = randomSeed
    return S_OK()


  def setArguments(self, args):
    """Define the arguments of the script.

    Alternative to :func:`DelphesApp.setExtraCLIArguments`.

    :param str args: Arguments to pass to the command call
    """
    self._checkArgs({'args': (str,)})
    self.extraCLIArguments = args
    return S_OK()
  
  def setExecutableName(self, executableName):
    """Set the executable.

    :param str executableName: Name of the delphes executable program.
    """
    self._checkArgs({'executableName': (str,)})
    self.executableName = executableName
    return S_OK()

  
  def setDetectorCard(self, detectorCard):
    """Set the detector card.

    :param str detectorCard: Name of the detector configuration file in Tcl format.
    """
    self._checkArgs({'detectorCard': (str,)})
    self.detectorCard = detectorCard
    if os.path.exists(detectorCard) or detectorCard.lower().count("lfn:"):
      self.inputSB.append(detectorCard)
    return S_OK()


  def setOutputCard(self, outputCard):
    """Set the output card.

    :param str outputCard: Name of the configuration file steering the content of the edm4hep output in Tcl format.
    """
    self._checkArgs({'outputCard': (str,)})
    self.outputCard = outputCard
    if os.path.exists(outputCard) or outputCard.lower().count("lfn:"):
      self.inputSB.append(outputCard)
    return S_OK()


  def setPythia8Card(self, pythia8CardPath):
    """Set the pythia8 card.

    :param str pythia8CardPath: Name of the Pythia8 configuration file path.
    """
    pythiaCardsLocations = self._ops.getValue("/ProcessList/PythiaCardsLocation", [])
    self._checkArgs({'pythia8CardPath': (str,)})

    if os.path.isfile(pythia8CardPath):
      self.pythia8CardContent = open(pythia8CardPath).read()
      return S_OK()

    # This is the LHE reader card, which is not present in the directory with all the standard pythia cards.
    # To avoid requiring the user to have that one card locally while all the others could be retrieved from EOS. 
    # We provide it here
    if pythia8CardPath in ['Pythia_LHEinput.cmd']:
      self.pythia8CardContent = PYTHIA_LHE_INPUT_CMD
      return S_OK()

    for PythiaCardsLocation in pythiaCardsLocations:
      pythia8CardEOS = os.path.join(PythiaCardsLocation, pythia8CardPath)
      if os.path.isfile(pythia8CardEOS):
        LOG.notice(f"Reading pythia card from {pythia8CardEOS}")
        self.pythia8CardContent = open(pythia8CardEOS).read()
        return S_OK()

    return self._reportError('Pythia8 configuration file does not exist!')


  def _userjobmodules(self, stepdefinition):
    res1 = self._setApplicationModuleAndParameters(stepdefinition)
    res2 = self._setUserJobFinalization(stepdefinition)
    if not res1["OK"] or not res2["OK"]:
      return S_ERROR('userjobmodules failed')
    return S_OK()

  def _prodjobmodules(self, stepdefinition):
    res1 = self._setApplicationModuleAndParameters(stepdefinition)
    res2 = self._setOutputComputeDataList(stepdefinition)
    if not res1["OK"] or not res2["OK"]:
      return S_ERROR('prodjobmodules failed')
    return S_OK()

  def _checkConsistency(self, job=None):
    """Check consistency of the DelphesApp application.

    Called from the `Job` instance.

    :param job: The instance of the job
    :type job: ~ILCDIRAC.Interfaces.API.NewInterface.Job.Job
    :returns: S_OK/S_ERROR
    """
    parameterName = [pN for pN in job.workflow.parameters.getParametersNames() if 'ConfigPackage' in pN]
    configversion = None
    if parameterName:
      LOG.notice("Found config parameter", parameterName)
      config = job.workflow.parameters.find(parameterName[0])
      configversion = config.value
    # Platform must always be defined
    platform = job.workflow.parameters.find("Platform").value

    if not self.version:
      return S_ERROR('No version found')
    
    if self.executableName not in ['DelphesPythia8_EDM4HEP', 'DelphesSTDHEP_EDM4HEP', 'DelphesROOT_EDM4HEP']:
      return S_ERROR('Executable not supported. Supported executables: DelphesPythia8_EDM4HEP, DelphesSTDHEP_EDM4HEP, DelphesROOT_EDM4HEP')
    
    if self.randomSeed < -1:
      return S_ERROR('Random Seed has to be equal or greater than -1')
    
    if self.detectorCard:
      if not self.detectorCard.endswith(".tcl"):
        return S_ERROR('Wrong name for the detector config file. Hint: they all end in ".tcl"')
      if not os.path.exists(self.detectorCard) and not self.detectorCard.lower().startswith("lfn:"):
        res = Exists(self.detectorCard, platform=platform, configversion=configversion)
        if not res['OK']:
          return res
    else:
      return S_ERROR('Missing detector config-file.')
    
    if self.outputCard:
      if not self.outputCard.endswith(".tcl"):
        return S_ERROR('Wrong name for the output config file. Hint: they all end in ".tcl"')
      if not os.path.exists(self.outputCard) and not self.outputCard.lower().startswith("lfn:"):
        res = Exists(self.outputCard, platform=platform, configversion=configversion)
        if not res['OK']:
          return res
    else:
      return S_ERROR('Missing output-config-file.')
            
    if self.executableName == "DelphesPythia8_EDM4HEP":
      if not self.pythia8CardContent:
        return S_ERROR('Missing Pythia 8 Card. The execution of DelphesPythia8_EDM4HEP would not succeed')

      else:
        res = preparePythia8Card(self.pythia8CardContent, 0, self.randomSeed, self.energy)
        if not res['OK']:
          return res
      
    if self._jobtype != 'User':
      self._listofoutput.append({"outputFile": "@{OutputFile}", "outputPath": "@{OutputPath}",
                                 "outputDataSE": '@{OutputSE}'})

    return S_OK()

  def _applicationModule(self):

    md1 = self._createModuleDefinition()
    md1.addParameter(Parameter("randomSeed", 0, "int", "", "", False, False,
                               "Random seed for the generator"))
    md1.addParameter(Parameter("executableName", "", "string", "", "", False, False,
                               "Name of the executable"))
    md1.addParameter(Parameter("detectorCard", "", "string", "", "", False, False,
                               "Flag that defines the detector card (e.g. ``delphes_card_IDEA.tcl``)"))    
    md1.addParameter(Parameter("outputCard", "", "string", "", "", False, False,
                               "Flag that defines the output card (e.g. ``edm4hep_output_config.tcl``)"))
    md1.addParameter(Parameter("pythia8CardContent", "", "string", "", "", False, False,
                               "Flag that defines the pythia8 card (e.g. ``path/p8_ee_ggqq_ecm91.cmd``)"))      
    md1.addParameter(Parameter("debug", False, "bool", "", "", False, False, "debug mode"))
    return md1

  def _applicationModuleValues(self, moduleinstance):

    moduleinstance.setValue("randomSeed", self.randomSeed)
    moduleinstance.setValue("executableName", self.executableName)
    moduleinstance.setValue("detectorCard", self.detectorCard)
    moduleinstance.setValue("outputCard", self.outputCard)
    moduleinstance.setValue("pythia8CardContent", self.pythia8CardContent)
    moduleinstance.setValue("debug", self.debug)

  def _checkWorkflowConsistency(self):
    return self._checkRequiredApp()

  def _resolveLinkedStepParameters(self, stepinstance):
    if isinstance(self._linkedidx, six.integer_types):
      self._inputappstep = self._jobsteps[self._linkedidx]
    if self._inputappstep:
      stepinstance.setLink("InputFile", self._inputappstep.getType(), "OutputFile")
    return S_OK()



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""module for splitting utilities and everything related to it."""


def addJobIndexToFilename(filename, jobIndex):
  """add the jobIndex number to the filename before the extension or replace %n with jobIndex.

  For example::

    'myOutputFile.root' --> 'myOutputFile_123.root'
    'myOutput_%n_File.root' --> 'myOutput_123_File.root'

  :param str filename: the original name of the file
  :param int jobIndex: the jobIndex number
  :returns: new filename with jobIndex
  """
  if '%n' in filename:
    filename = filename.replace('%n', str(jobIndex))
    return filename

  fileParts = filename.rsplit('.', 1)
  if len(fileParts) == 2:
    filename = "%s_%d.%s" % (fileParts[0], jobIndex, fileParts[1])
    return filename

  filename = "%s_%d" % (filename, jobIndex)
  return filename



#!/usr/bin/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test the GaudiApp WorkflowModule."""

from __future__ import print_function
from __future__ import absolute_import
import unittest
import os
import os.path
import shutil
import tempfile
from mock import patch, MagicMock as Mock, mock_open

from parameterized import parameterized, param

from DIRAC import gLogger, S_OK, S_ERROR
from ILCDIRAC.Workflow.Modules.GaudiAppModule import GaudiAppModule
from Tests.Utilities.GeneralUtils import assertDiracSucceeds, assertDiracFails

__RCSID__ = "$Id$"


MODULE_NAME = 'ILCDIRAC.Workflow.Modules.GaudiAppModule'
MODULEBASE_NAME = 'ILCDIRAC.Workflow.Modules.ModuleBase'
PROXYINFO_NAME = 'DIRAC.Core.Security.ProxyInfo'
# pylint: disable=too-many-public-methods, protected-access

gLogger.setLevel("ERROR")
gLogger.showHeaders(True)


def cleanup(tempdir):
  """Remove files after run."""
  try:
    shutil.rmtree(tempdir)
  except OSError:
    pass


@patch("%s.getProxyInfoAsString" % MODULEBASE_NAME, new=Mock(return_value=S_OK()))
@patch("%s.getProxyInfoAsString" % PROXYINFO_NAME, new=Mock(return_value=S_OK()))
class TestGaudiAppModule(unittest.TestCase):
  """test GaudiAppModule."""

  def assertIn(self, *args, **kwargs):
    """make this existing to placate pylint."""
    return super(TestGaudiAppModule, self).assertIn(*args, **kwargs)

  @patch("%s.getProxyInfoAsString" % MODULEBASE_NAME, new=Mock(return_value=S_OK()))
  @patch("%s.getProxyInfoAsString" % PROXYINFO_NAME, new=Mock(return_value=S_OK()))
  def setUp(self):
    self.gaudi = GaudiAppModule()
    self.curdir = os.getcwd()
    self.tempdir = tempfile.mkdtemp("", dir="./")
    os.chdir(self.tempdir)
    self.gaudi.ops = Mock()

  def tearDown(self):
    os.chdir(self.curdir)
    cleanup(self.tempdir)

#---------------------------------------------------------------------------

class TestGaudiAppModuleRunit(TestGaudiAppModule):
  """test GaudiApp runtIt."""

  def setUp(self):
    super(TestGaudiAppModuleRunit, self).setUp()
    self.logFileName = "localEnv.log"
    self.executableName = "k4run"
    self.outputFileFlag = "flag"
    with open(self.logFileName, "w") as logF:
      logF.write("logged the logging logs")

#-------------------------- RUNIT - noscript -failures

  def test_GaudiApp_runIt_failure_NoPlatform(self):
    """GaudiApp.runit failure with platform ........................................................."""
    self.gaudi.applicationLog = self.logFileName
    self.gaudi.executableName = self.executableName
    res = self.gaudi.runIt()
    self.assertIn("No ILC platform selected", res['Message'])

  def test_GaudiApp_runIt_failure_NoLogFile(self):
    """gaudi.runit failure with logfile................................................................"""
    self.gaudi.platform = "Windows"
    self.gaudi.executableName = self.executableName
    res = self.gaudi.runIt()
    self.assertIn("No Log file provided", res['Message'])

  def test_GaudiApp_runIt_failure_NoExecutable(self):
    """gaudi.runit failure with executable name................................................................"""
    self.gaudi.platform = "Windows"
    self.gaudi.applicationLog = self.logFileName
    res = self.gaudi.runIt()
    self.assertIn("No executable name provided", res['Message'])

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_ERROR("missing setup.sh")))
  def test_GaudiApp_runIt_failure_env(self):
    """gaudi.runit failed to get env................................................................"""
    self.gaudi.platform = "Windows"
    self.gaudi.applicationLog = self.logFileName
    self.gaudi.executableName = self.executableName
    res = self.gaudi.runIt()
    self.assertEqual(res['Message'], "missing setup.sh")

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value={"OK": False, "Message": ""}))
  def test_GaudiApp_runIt_failure_wrong_env_res(self):
    """gaudi.runit failed to obtain the environment script................................................................"""
    self.gaudi.platform = "Windows"
    self.gaudi.applicationLog = self.logFileName
    self.gaudi.executableName = self.executableName
    res = self.gaudi.runIt()
    assertDiracFails(res, self)

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("setup.sh")))
  def test_GaudiApp_runIt_failure_WrongSteeringFileName(self):
    """GaudiApp.runit failure with the name of a steering file that does not exist............................................."""
    self.gaudi.platform = "Windows"
    self.gaudi.executableName = self.executableName
    self.gaudi.applicationLog = self.logFileName
    self.gaudi.SteeringFile = "steeringfilethatisnotthere"
    # side effect for the os.path.exists in GaudiAppModule:
    # The first one (self.SteeringFile) (in GaudiAppModule) was called.
    # self.SteeringFile, scriptName, "lib", self.applicationLog, neventsfile, self.applicationLog
    # actually the error happens before the last 5 ones are used   
    with patch("os.path.exists", new=Mock(side_effect=[False, False, False, False, False, False])):
      res = self.gaudi.runIt()
    self.assertIn("Could not find steering file", res['Message'])

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("setup.sh")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  def test_GaudiApp_runIt_failure_LogFile(self):
    """GaudiApp.runit failure with applicationLog......................................................"""
    self.gaudi.platform = "Windows"
    self.gaudi.applicationLog = self.logFileName
    self.gaudi.executableName = self.executableName
    self.gaudi.ignoreapperrors = False
    self.gaudi.outputFileFlag= self.outputFileFlag
    # side effect for the os.path.exists in GaudiAppModule:
    # The first one (self.SteeringFile) (in GaudiAppModule) was not called. 5 were left:
    # scriptName, "lib", self.applicationLog, neventsfile, self.applicationLog
    with patch("os.path.exists", new=Mock(side_effect=[False, False, False, False, False])):
      res = self.gaudi.runIt()
    self.assertIn("did not produce the expected log", res['Message'])

#-------------------------- RUNIT - noscript - success

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("setup.sh")))
  @patch("%s.resolveIFpaths" % MODULE_NAME, new=Mock(return_value={"OK": True, "Value": "inputfile", "Message": "just a test"}))
  def test_GaudiApp_runIt_success_inputfile_ignoreapperrors(self):
    """gaudi.runit failure using the resolveIFpaths function but changing its output................................................................"""
    self.gaudi.platform = "Windows"
    self.gaudi.applicationLog = self.logFileName
    self.gaudi.executableName = self.executableName
    self.gaudi.InputFile = "inputfile"
    self.gaudi.compactFile = "compactfile"
    self.gaudi.NumberOfEvents = "100"
    self.gaudi.ignoreapperrors = True
    with patch("os.path.exists", new=Mock(side_effect=[False, False, True, False, True])):
      res = self.gaudi.runIt()
      assertDiracSucceeds(res, self)

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("setup.sh")))
  def test_GaudiApp_runIt_success_MockedSteeringFileName_ignoreapperrors(self):
    """GaudiApp.runit failure with the name of a steering file that does not exist............................................."""
    self.gaudi.platform = "Windows"
    self.gaudi.executableName = self.executableName
    self.gaudi.applicationLog = self.logFileName
    self.gaudi.SteeringFile = "steeringfilettobemocked"
    self.gaudi.ignoreapperrors = True
    # side effect for the os.path.exists in GaudiAppModule:
    # The first one (self.SteeringFile) (in GaudiAppModule) was called.
    # self.SteeringFile, scriptName, "lib", self.applicationLog, neventsfile, self.applicationLog
    # actually the error happens before the last 5 ones are used   
    with patch("os.path.exists", new=Mock(side_effect=[True, False, False, False, False, False])):
      with patch("os.path.basename", new=Mock(return_value=True)):
        res = self.gaudi.runIt()
        assertDiracSucceeds(res, self)

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("setup.sh")))
  def test_GaudiApp_runIt_success_workflowStatusOK(self):
    """gaudi.runit success while having workflowStatus[OK] set as False................................................................"""
    self.gaudi.platform = "Windows"
    self.gaudi.applicationLog = self.logFileName
    self.gaudi.executableName = self.executableName
    self.gaudi.workflowStatus['OK'] = False
    #NB: workflowStatus['OK'] = False will cause RunIt() to be interrupted earlier
    res = self.gaudi.runIt()
    assertDiracSucceeds(res, self)

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("setup.sh")))
  @patch("%s.resolveIFpaths" % MODULE_NAME, new=Mock(return_value={"OK": False, "Message": "just a test"}))
  def test_GaudiApp_runIt_success_resolveIFpaths(self):
    """gaudi.runit success using the resolveIFpaths function................................................................"""
    self.gaudi.platform = "Windows"
    self.gaudi.applicationLog = self.logFileName
    self.gaudi.executableName = self.executableName
    self.gaudi.InputFile = "inputfile"
    res = self.gaudi.runIt()
    #NB: resolveIFpaths = {"OK": False} will cause RunIt() to be interrupted earlier
    self.assertEqual(res['Message'], "just a test")

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("setup.sh")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  def test_GaudiApp_runIt_fail_getdetectormodelmocking(self):
    """gaudi.runit fail using the getdetectormodel function but mocked................................................................"""
    self.gaudi.platform = "Windows"
    self.gaudi.applicationLog = self.logFileName
    self.gaudi.executableName = self.executableName
    self.gaudi.detectorModel = "detectormodel"
    with patch("ILCDIRAC.Workflow.Utilities.DD4hepMixin.DD4hepMixin._getDetectorXML", new=Mock(return_value={"OK": True, "Value": "something"})):
      res = self.gaudi.runIt()
    self.assertIn("gaudiapp did not produce the expected log", res['Message'])

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("setup.sh")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  def test_GaudiApp_runIt_failure_LogFile_ignoreapperrors(self):
    """GaudiApp.runit failure with applicationLog but ignore..........................................."""
    self.gaudi.platform = "Windows"
    self.gaudi.applicationLog = self.logFileName
    self.gaudi.executableName = self.executableName
    self.gaudi.ignoreapperrors = True
    self.gaudi.outputFileFlag = self.outputFileFlag
    # side effect for the os.path.exists in GaudiAppModule:
    # The first one (self.SteeringFile) (in GaudiAppModule) was not called. 5 were left:
    # scriptName, "lib", self.applicationLog, neventsfile, self.applicationLog
    with patch("os.path.exists", new=Mock(side_effect=[False, False, False, False, False])):
      res = self.gaudi.runIt()
    assertDiracSucceeds(res, self)

#--------------------------------RUNIT - withscript

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("setup.sh")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  def test_GaudiApp_runIt_success_LogAndScriptPresent(self):
    """GaudiApp.runit success log and script exist..................................................."""
    self.gaudi.platform = "Windows"
    self.gaudi.applicationLog = self.logFileName
    self.gaudi.ignoreapperrors = True
    self.gaudi.executableName = self.executableName
    self.gaudi.outputFileFlag= self.outputFileFlag
    with open("GaudiApp__Run_.sh", "w") as scr:
      scr.write("content")
    with open("GaudiApp__Steer_.input", "w") as scr:
      scr.write("content")
    with open(self.logFileName, "w") as scr:
      scr.write("content")
    # side effect for the os.path.exists in GaudiAppModule:
    # The first one (self.SteeringFile) (in GaudiAppModule) was not called. 5 were left:
    # scriptName, "lib", self.applicationLog, neventsfile, self.applicationLog    with patch("os.path.exists", new=Mock(side_effect=[True, True, False, True, True])):#[True, True, False, True, False]
    with patch("os.path.exists", new=Mock(side_effect=[True, True, False, True, False])):
      res = self.gaudi.runIt()
    assertDiracSucceeds(res, self)

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("setup.sh")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  def test_GaudiApp_runIt_success_LogAndScriptPresentandrandomseedflag(self):
    """GaudiApp.runit success log and script exist..................................................."""
    self.gaudi.platform = "Windows"
    self.gaudi.applicationLog = self.logFileName
    self.gaudi.ignoreapperrors = True
    self.gaudi.executableName = self.executableName
    self.gaudi.outputFileFlag= self.outputFileFlag
    self.gaudi.randomSeedFlag= '--SimG4Svc.seedValue'
    with open("GaudiApp__Run_.sh", "w") as scr:
      scr.write("content")
    with open("GaudiApp__Steer_.input", "w") as scr:
      scr.write("content")
    with open(self.logFileName, "w") as scr:
      scr.write("content")
    # side effect for the os.path.exists in GaudiAppModule:
    # The first one (self.SteeringFile) (in GaudiAppModule) was not called. 5 were left:
    # scriptName, "lib", self.applicationLog, neventsfile, self.applicationLog    with patch("os.path.exists", new=Mock(side_effect=[True, True, False, True, True])):#[True, True, False, True, False]
    with patch("os.path.exists", new=Mock(side_effect=[True, True, False, True, False])):
      res = self.gaudi.runIt()
    assertDiracSucceeds(res, self)

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("setup.sh")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  def test_GaudiApp_runIt_success_pythiacard(self):
    """GaudiApp.runit success using the pythia card..................................................."""
    self.gaudi.platform = "Windows"
    self.gaudi.applicationLog = self.logFileName
    self.gaudi.ignoreapperrors = True
    self.gaudi.executableName = self.executableName
    self.gaudi.outputFileFlag = self.outputFileFlag
    self.gaudi.randomSeedFlag = '--SimG4Svc.seedValue'
    self.gaudi.pythia8CardContent = "Beams:LHEF\nMain:numberOfEvents"
    with open("GaudiApp__Run_.sh", "w") as scr:
      scr.write("content")
    with open("GaudiApp__Steer_.input", "w") as scr:
      scr.write("content")
    with open(self.logFileName, "w") as scr:
      scr.write("content")
    # side effect for the os.path.exists in GaudiAppModule:
    # The first one (self.SteeringFile) (in GaudiAppModule) was not called. 5 were left:
    # scriptName, "lib", self.applicationLog, neventsfile, self.applicationLog    with patch("os.path.exists", new=Mock(side_effect=[True, True, False, True, True])):#[True, True, False, True, False]
    with patch("os.path.exists", new=Mock(side_effect=[True, True, False, True, False])):
      res = self.gaudi.runIt()
    assertDiracSucceeds(res, self)

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("setup.sh")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  def test_GaudiApp_runIt_failure_notfindbeam(self):
    """GaudiApp.runit not succeeded in finding beam in pythia card..................................................."""
    self.gaudi.platform = "Windows"
    self.gaudi.applicationLog = self.logFileName
    self.gaudi.ignoreapperrors = True
    self.gaudi.executableName = self.executableName
    self.gaudi.outputFileFlag = self.outputFileFlag
    self.gaudi.randomSeedFlag = '--SimG4Svc.seedValue'
    self.gaudi.pythia8CardContent = "ciao\nMain:numberOfEvents\nciao"
    with open("GaudiApp__Run_.sh", "w") as scr:
      scr.write("content")
    with open("GaudiApp__Steer_.input", "w") as scr:
      scr.write("content")
    with open(self.logFileName, "w") as scr:
      scr.write("content")
    # side effect for the os.path.exists in GaudiAppModule:
    # The first one (self.SteeringFile) (in GaudiAppModule) was not called. 5 were left:
    # scriptName, "lib", self.applicationLog, neventsfile, self.applicationLog    with patch("os.path.exists", new=Mock(side_effect=[True, True, False, True, True])):#[True, True, False, True, False]
    with patch("os.path.exists", new=Mock(side_effect=[True, True, False, True, False])):
      res = self.gaudi.runIt()
    self.assertIn("Pythia card with unusual content: neither `Beams:LHEF` nor `Beams:eCM` fields present. Please check the content of your card.", res['Message'])

#--------------------------------ASI

class TestGaudiAppModuleASI(TestGaudiAppModule):
  """gaudi.ApplicationSpecificInputs."""

  @patch.dict(os.environ, {"JOBID": "12345"})
  def test_GaudiApp_ASI_NoVariables(self):
    """gaudi.applicationSpecificInputs: checks that no variables have been set after this call......"""
    gLogger.setLevel("ERROR")
    self.gaudi.workflow_commons = dict()
    self.gaudi.applicationSpecificInputs()
    self.assertFalse(self.gaudi.jobReport or self.gaudi.productionID)

  @patch.dict(os.environ, {"JOBID": "12345"})
  def test_GaudiAPP_ASI_success_workflowstartfrom_Inputdata(self):
    """GaudiApp.applicationSpecificInputs success with a workflowstartfrom............................................."""
    self.gaudi.InputData = "inputdata"
    self.gaudi.WorkflowStartFrom = True     
    res = self.gaudi.applicationSpecificInputs()
    assertDiracSucceeds(res, self)

  @patch.dict(os.environ, {"JOBID": "12345"})
  def test_GaudiAPP_ASI_success_Randomseed(self):
    """GaudiApp.applicationSpecificInputs success with a randomseed............................................."""
    self.gaudi.workflow_commons = {"IS_PROD": True,
                                     "PRODUCTION_ID": '2345',
                                     "JOB_ID": '12345'}
    self.gaudi.OutputFile = 'outputfile.root'
    self.gaudi.randomSeed = 10
    res = self.gaudi.applicationSpecificInputs()
    assertDiracSucceeds(res, self)

#-----------------------------------------------------------------------------------------

def runTests():
  """Runs our tests."""
  suite = unittest.defaultTestLoader.loadTestsFromTestCase(TestGaudiAppModule)
  suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(TestGaudiAppModuleRunit))
  suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(TestGaudiAppModuleASI))
  testResult = unittest.TextTestRunner(verbosity=2).run(suite)
  print(testResult)


#if __name__ == '__main__':
runTests()



#!/usr/local/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test GaudiApp module."""

from __future__ import print_function
from __future__ import absolute_import
import linecache
import unittest
from mock import patch, MagicMock as Mock
from mock import mock_open
from mock import mock as mock_module
from parameterized import parameterized
import six
import os, shutil, tempfile

from DIRAC import gLogger, S_OK, S_ERROR
from ILCDIRAC.Interfaces.API.NewInterface.Applications import GaudiApp
from Tests.Utilities.GeneralUtils import assertEqualsImproved, assertDiracFailsWith, \
    assertDiracSucceeds

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Interfaces.API.NewInterface.Applications.GaudiApp'
BUILTIN_NAME = 'builtins' if six.PY3 else '__builtin__'

gLogger.setLevel("DEBUG")
gLogger.showHeaders(True)

# pylint: disable=protected-access
def cleanup(tempdir):
  """Remove files after run."""
  try:
    shutil.rmtree(tempdir)
  except OSError:
    pass

class GaudiAppTestCase(unittest.TestCase):
  """Base class for the GaudiApp test cases."""

  @classmethod
  def setUpClass(cls):
    """Load the Application file into the linecache to prevent exceptions when mocking the builtin open."""
    from ILCDIRAC.Interfaces.API.NewInterface import Application
    for fName in [Application.__file__, mock_module.__file__]:
      if fName.endswith(('.pyc', '.pyo')):
        fName = fName[:-1]
      linecache.getlines(fName)

  @classmethod
  def tearDownClass(cls):
    """Remove all entries from linecache because we mock builtin open."""
    linecache.clearcache()

  def setUp(self):
    """set up the objects."""
    self.gaudi = GaudiApp({})
    self.gaudi._jobtype = 'User'
    self.gaudi._ops = Mock(name='OpsMock')
    self.curdir = os.getcwd()
    self.tempdir = tempfile.mkdtemp("", dir="./")
    os.chdir(self.tempdir)

  def tearDown(self):
    os.chdir(self.curdir)
    cleanup(self.tempdir)
    
  def test_setrandomseed(self):
    self.assertFalse(self.gaudi._errorDict)
    self.gaudi.setRandomSeed(89421)
    self.assertFalse(self.gaudi._errorDict)
    assertEqualsImproved(self.gaudi.randomSeed, 89421, self)

  def test_setrandomseed_fails(self):
    self.assertFalse(self.gaudi._errorDict)
    self.gaudi.setRandomSeed(['abc'])
    self.assertIn('_checkArgs', self.gaudi._errorDict)

  def test_setrandomseedflag(self):
    self.assertFalse(self.gaudi._errorDict)
    self.gaudi.setRandomSeedFlag("randomseedflag")
    self.assertFalse(self.gaudi._errorDict)
    assertEqualsImproved(self.gaudi.randomSeedFlag, "randomseedflag", self)

  def test_setrandomseedflag_fails(self):
    self.assertFalse(self.gaudi._errorDict)
    self.gaudi.setRandomSeedFlag(None)
    self.assertIn('_checkArgs', self.gaudi._errorDict)

  def test_setgaudiworkflow(self):
    self.assertFalse(self.gaudi._errorDict)
    self.gaudi.setGaudiWorkFlow("fullsim")
    self.assertFalse(self.gaudi._errorDict)
    assertEqualsImproved(self.gaudi.gaudiWorkFlow, "fullsim", self)

  def test_setgaudiworkflow_fails(self):
    self.assertFalse(self.gaudi._errorDict)
    self.gaudi.setGaudiWorkFlow(None)
    self.assertIn('_checkArgs', self.gaudi._errorDict)
    
  def test_setinputfileflag(self):
    self.assertFalse(self.gaudi._errorDict)
    self.gaudi.setInputFileFlag("inputfileflag")
    self.assertFalse(self.gaudi._errorDict)
    assertEqualsImproved(self.gaudi.inputFileFlag, "inputfileflag", self)

  def test_setinputfileflag_fails(self):
    self.assertFalse(self.gaudi._errorDict)
    self.gaudi.setInputFileFlag(None)
    self.assertIn('_checkArgs', self.gaudi._errorDict)

  def test_setoutputfileflag(self):
    self.assertFalse(self.gaudi._errorDict)
    self.gaudi.setOutputFileFlag("outputfileflag")
    self.assertFalse(self.gaudi._errorDict)
    assertEqualsImproved(self.gaudi.outputFileFlag, "outputfileflag", self)

  def test_setoutputfileflag_fails(self):
    self.assertFalse(self.gaudi._errorDict)
    self.gaudi.setOutputFileFlag(None)
    self.assertIn('_checkArgs', self.gaudi._errorDict)

  def test_setdetectormodelflag(self):
    self.assertFalse(self.gaudi._errorDict)
    self.gaudi.setDetectorModelFlag("detectormodelflag")
    self.assertFalse(self.gaudi._errorDict)
    assertEqualsImproved(self.gaudi.detectorModelFlag, "detectormodelflag", self)

  def test_setdetectormodelflag_fails(self):
    self.assertFalse(self.gaudi._errorDict)
    self.gaudi.setDetectorModelFlag(None)
    self.assertIn('_checkArgs', self.gaudi._errorDict)

  def test_setexecutable(self):
    self.assertFalse(self.gaudi._errorDict)
    self.gaudi.setExecutableName("executable")
    self.assertFalse(self.gaudi._errorDict)
    assertEqualsImproved(self.gaudi.executableName, "executable", self)

  def test_setexecutable_fails(self):
    self.assertFalse(self.gaudi._errorDict)
    self.gaudi.setExecutableName(None)
    self.assertIn('_checkArgs', self.gaudi._errorDict)

  def test_setstartfrom(self):
    self.assertFalse(self.gaudi._errorDict)
    self.gaudi.setStartFrom(89421)
    self.assertFalse(self.gaudi._errorDict)
    assertEqualsImproved(self.gaudi.startFrom, 89421, self)

  def test_setstartfrom_fails(self):
    self.assertFalse(self.gaudi._errorDict)
    self.gaudi.setStartFrom('adgiuj')
    self.assertIn('_checkArgs', self.gaudi._errorDict)

  def test_keeprecfile(self):
    self.assertFalse(self.gaudi._errorDict)
    self.gaudi.setKeepRecFile(True)
    self.assertFalse(self.gaudi._errorDict)
    assertEqualsImproved(self.gaudi.keepRecFile, True, self)

  def test_keeprecfile_fails(self):
    self.assertFalse(self.gaudi._errorDict)
    self.gaudi.setKeepRecFile(89421)
    self.assertIn('_checkArgs', self.gaudi._errorDict)

  def test_setPythia8Card(self):
    self.assertFalse(self.gaudi._errorDict)
    with patch("builtins.open", mock_open(read_data='content')), \
         patch('os.path.isfile', new=Mock(return_value=True)):
      res = self.gaudi.setPythia8Card("pythia8card.cmd")
      assertDiracSucceeds(res, self)
      assertEqualsImproved(self.gaudi.pythia8CardContent, 'content', self)

  def test_setPythiaLHEreader(self):
    self.assertFalse(self.gaudi._errorDict)
    res = self.gaudi.setPythia8Card("Pythia_LHEinput.cmd")
    assertDiracSucceeds(res, self)
    self.assertIn('! 2) Settings related to output in init(), next() and stat().', self.gaudi.pythia8CardContent)

  def test_setPythia8CardEOS(self):
    self.assertFalse(self.gaudi._errorDict)
    with patch("builtins.open", mock_open(read_data='content')), \
         patch("os.path.isfile", new=Mock(side_effect=[False, True])), \
         patch.object(self.gaudi._ops, 'getValue', return_value=['firstlocation']):
      res = self.gaudi.setPythia8Card("p8_ee_tt_ecm365.cmd")
      assertDiracSucceeds(res, self)
      assertEqualsImproved(self.gaudi.pythia8CardContent, 'content', self)

  def test_setPythia8Card_fails_file(self):
    self.assertFalse(self.gaudi._errorDict)
    res = self.gaudi.setPythia8Card("content")
    assertDiracFailsWith(res, 'Pythia8 configuration file does not exist!', self)

  def test_setPythia8Card_fails_argument(self):
    self.assertFalse(self.gaudi._errorDict)
    res = self.gaudi.setPythia8Card(1)
    self.assertIn('_checkArgs', self.gaudi._errorDict)

  def test_setPythia8CardEOS_fails(self):
    self.assertFalse(self.gaudi._errorDict)
    with patch("builtins.open", mock_open(read_data='content')), \
         patch("os.path.isfile", new=Mock(side_effect=[False, False])), \
         patch.object(self.gaudi._ops, 'getValue', return_value=['firstlocation']):
      res = self.gaudi.setPythia8Card("p8_ee_tt_ecm365.cmd")
      assertDiracFailsWith(res, 'Pythia8 configuration file does not exist!', self)

  def test_setarguments(self):
    self.assertFalse(self.gaudi._errorDict)
    self.gaudi.setArguments("arguments")
    self.assertFalse(self.gaudi._errorDict)
    assertEqualsImproved(self.gaudi.extraCLIArguments, "arguments", self)
    assertDiracSucceeds(self.gaudi.setArguments("arguments"), self)

  def test_setarguments_fails(self):
    self.assertFalse(self.gaudi._errorDict)
    self.gaudi.setArguments(None)
    self.assertIn('_checkArgs', self.gaudi._errorDict)

  def test_setrecfilepath(self):
    self.assertFalse(self.gaudi._errorDict)
    self.gaudi.setOutputRecFile("name", "path")
    self.assertFalse(self.gaudi._errorDict)
    assertEqualsImproved(self.gaudi.outputRecFile, "name", self)
    assertEqualsImproved(self.gaudi.outputRecPath, "path", self)

  def test_setrecfilepath_fails(self):
    self.assertFalse(self.gaudi._errorDict)
    self.gaudi.setOutputRecFile(89421)
    self.assertIn('_checkArgs', self.gaudi._errorDict)

  def test_setsimfilepath(self):
    self.assertFalse(self.gaudi._errorDict)
    self.gaudi.setOutputSimFile("name", "path")
    self.assertFalse(self.gaudi._errorDict)
    assertEqualsImproved(self.gaudi.outputSimFile, "name", self)
    assertEqualsImproved(self.gaudi.outputSimPath, "path", self)

  def test_setsimfilepath_fails(self):
    self.assertFalse(self.gaudi._errorDict)
    self.gaudi.setOutputSimFile(89421)
    self.assertIn('_checkArgs', self.gaudi._errorDict)
    
#--------------------------------------------------------------------------

  def test_resolvelinkedparams(self):
    step_mock = Mock()
    input_mock = Mock()
    input_mock.getType.return_value = {'abc': False}
    self.gaudi._linkedidx = 3
    self.gaudi._jobsteps = [None, None, None, input_mock]
    assertDiracSucceeds(self.gaudi._resolveLinkedStepParameters(step_mock), self)
    step_mock.setLink.assert_called_once_with('InputFile', {'abc': False}, 'OutputFile')

  def test_resolvelinkedparams_noinputstep(self):
    self.gaudi._linkedidx = None
    self.gaudi._inputappstep = []
    assertDiracSucceeds(self.gaudi._resolveLinkedStepParameters(None), self)

  def test_checkworkflow_app_missing(self):
    self.gaudi._inputapp = ['some_depdency', 'unavailable_dependency_fail_on_this']
    self.gaudi._jobapps = ['myjobapp_1', 'some_dependency']
    assertDiracFailsWith(self.gaudi._checkWorkflowConsistency(), 'job order not correct', self)

  def test_checkworkflow_empty(self):
    self.gaudi._inputapp = []
    self.gaudi._jobapps = []
    assertDiracSucceeds(self.gaudi._checkWorkflowConsistency(), self)

  def test_checkworkflow_success(self):
    self.gaudi._inputapp = ['some_dependency', 'other_dependencies', 'many_more']
    self.gaudi._jobapps = ['ignore_me', 'many_more', 'some_dependency', 'other_dependencies']
    assertDiracSucceeds(self.gaudi._checkWorkflowConsistency(), self)

  def test_userjobmodules(self):
    module_mock = Mock()
    assertDiracSucceeds(self.gaudi._userjobmodules(module_mock), self)

  def test_prodjobmodules(self):
    module_mock = Mock()
    assertDiracSucceeds(self.gaudi._prodjobmodules(module_mock), self)

  def test_userjobmodules_fails(self):
    with patch('%s._setUserJobFinalization' % MODULE_NAME, new=Mock(return_value=S_OK('something'))),\
            patch('%s._setApplicationModuleAndParameters' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_test_err'))):
      assertDiracFailsWith(self.gaudi._userjobmodules(None),
                           'userjobmodules failed', self)

  def test_prodjobmodules_fails(self):
    with patch('%s._setApplicationModuleAndParameters' % MODULE_NAME, new=Mock(return_value=S_OK('something'))), \
            patch('%s._setOutputComputeDataList' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_other_test_err'))):
      assertDiracFailsWith(self.gaudi._prodjobmodules(None),
                           'prodjobmodules failed', self)

#----------------------------------------------------------------------------

  def test_checkconsistency_notuserjobtype(self):
    self.gaudi.version = '134'
    self.gaudi.detectorModel = 'mymodel.det'
    self.gaudi.outputFile = 'myoutput.file'
    self.gaudi._jobtype = 'NotaUser'
    assertDiracSucceeds(self.gaudi._checkConsistency(Mock()), self)
    self.assertIn({'outputFile': '@{OutputFile}', 'outputPath': '@{OutputPath}',
                        'outputDataSE': '@{OutputSE}'}, self.gaudi._listofoutput)
    self.assertNotIn('nbevts', self.gaudi.prodparameters)
    self.assertNotIn('Process', self.gaudi.prodparameters)

  def test_checkconsistency_notStartFrom(self):
    self.gaudi.version = '134'
    self.gaudi.startFrom = True
    self.gaudi.detectorModel = 'mymodel.det'
    self.gaudi.outputFile = 'myoutput.file'
    self.gaudi._jobtype = 'User'
    assertDiracSucceeds(self.gaudi._checkConsistency(Mock()), self)
    self.assertNotIn({'outputFile': '@{OutputFile}', 'outputPath': '@{OutputPath}',
                        'outputDataSE': '@{OutputSE}'}, self.gaudi._listofoutput)
    self.assertNotIn('nbevts', self.gaudi.prodparameters)
    self.assertNotIn('Process', self.gaudi.prodparameters)

  def test_checkconsistency_noversion(self):
    self.gaudi.version = None
    assertDiracFailsWith(self.gaudi._checkConsistency(Mock()), 'no version found', self)

  def test_checkconsistency_lfn_steeringfile(self):
    self.gaudi.version = '134'
    self.gaudi.detectorModel = 'mymodel.det'
    self.gaudi.outputFile = 'myoutput.file'
    self.gaudi._jobtype = 'User'
    self.gaudi.steeringFile = 'lfn:steeringFile'
    assertDiracSucceeds(self.gaudi._checkConsistency(Mock()), self)
    self.assertNotIn({'outputFile': '@{OutputFile}', 'outputPath': '@{OutputPath}',
                        'outputDataSE': '@{OutputSE}'}, self.gaudi._listofoutput)
    self.assertNotIn('nbevts', self.gaudi.prodparameters)
    self.assertNotIn('Process', self.gaudi.prodparameters)

  def test_checkconsistency_okaysteeringfile(self):
    self.gaudi.version = '134'
    self.gaudi.detectorModel = 'mymodel.det'
    self.gaudi.outputFile = 'myoutput.file'
    self.gaudi._jobtype = 'User'
    self.gaudi.steeringFile = 'geant_fullsim_fccee_lar_pgun.py'
    assertDiracSucceeds(self.gaudi._checkConsistency(Mock()), self)
    self.assertNotIn({'outputFile': '@{OutputFile}', 'outputPath': '@{OutputPath}',
                        'outputDataSE': '@{OutputSE}'}, self.gaudi._listofoutput)
    self.assertNotIn('nbevts', self.gaudi.prodparameters)
    self.assertNotIn('Process', self.gaudi.prodparameters)

  @patch("DIRAC.Core.Workflow.Parameter.ParameterCollection.getParametersNames", new=Mock(return_value={"a":1, "b":2, "c":3}))
  def test_checkconsistency_badsteeringfile(self):
    self.gaudi.version = '134'
    self.gaudi.detectorModel = 'mymodel.det'
    self.gaudi.outputFile = 'myoutput.file'
    self.gaudi._jobtype = 'User'
    self.gaudi.steeringFile = 'badsteeringFile'
    self.assertIn('is not available locally nor in the software installation', \
                  str(self.gaudi._checkConsistency(Mock())))
    
  def test_checkconsistency_badpythia8card(self):
    self.gaudi.version = '134'
    self.gaudi.detectorModel = 'mymodel.det'
    self.gaudi.outputFile = 'myoutput.file'
    self.gaudi._jobtype = 'User'
    self.gaudi.steeringFile = 'lfn:steeringFile'
    self.gaudi.pythia8CardContent = 'poorcontent'
    self.gaudi.gaudiWorkFlow = 'fastsim'
    res = self.gaudi._checkConsistency(Mock())
    assertDiracFailsWith(self.gaudi._checkConsistency(Mock()), 'Pythia card with unusual content: neither `Beams:LHEF` nor `Beams:eCM` fields present. Please check the content of your card.', self)

    
  def test_checkconsistency_goodpythia8card(self):
    self.gaudi.version = '134'
    self.gaudi.detectorModel = 'mymodel.det'
    self.gaudi.outputFile = 'myoutput.file'
    self.gaudi._jobtype = 'User'
    self.gaudi.steeringFile = 'lfn:steeringFile'
    self.gaudi.pythia8CardContent = 'Beams:LHEF\nMain:numberOfEvents'
    assertDiracSucceeds(self.gaudi._checkConsistency(Mock()), self)


#----------------------------------------------------------------------------

def runTests():
  """Runs our tests."""
  suite = unittest.defaultTestLoader.loadTestsFromTestCase(GaudiAppTestCase)
  testResult = unittest.TextTestRunner(verbosity=2).run(suite)
  print(testResult)


if __name__ == '__main__':
  runTests()



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""
StdHepSplit : Helper to split Stdhep files
"""

from __future__ import absolute_import
import types

from DIRAC.Core.Workflow.Parameter import Parameter
from DIRAC import S_OK, S_ERROR, gLogger

from ILCDIRAC.Interfaces.API.NewInterface.LCUtilityApplication import LCUtilityApplication
import six

LOG = gLogger.getSubLogger(__name__)

__RCSID__ = "$Id$"


class StdHepSplit(LCUtilityApplication):
  """Helper to split stdhep files.

  Example:

  >>> stdhepsplit = StdHepSplit()
  >>> stdhepsplit.setInputFile( "File_1.stdhep" )
  >>> stdhepsplit.setNumberOfEventsPerFile(100)
  >>> stdhepsplit.setOutputFile("somefile.stdhep")

  The outpufiles will then be *somefile_X.stdhep*, where *X* corresponds to the slice index
  """

  def __init__(self, paramdict=None):
    self.numberOfEventsPerFile = 0
    self.maxRead = 0
    super(StdHepSplit, self).__init__(paramdict)
    if not self.version:
      self.version = 'V2'
    self._modulename = "StdHepSplit"
    self.appname = 'stdhepsplit'
    self._moduledescription = 'Helper call to split Stdhep files'

  def setNumberOfEventsPerFile(self, numberofevents):
    """Number of events to have in each file.

    :param int numberofevents: number of events in each output file
    """
    self._checkArgs({'numberofevents': int})
    self.numberOfEventsPerFile = numberofevents

  def setMaxRead(self, maxRead):
    """set the number of events to get out of the input stdhep file.

    :param int maxRead: number of events to get out of the input stdhep file

    .. note::

      We already correct for the off-by-1 error of hepslit by using maxRead+1, which results in maxRead events read.
    """
    self._checkArgs({'maxRead': int})
    self.maxRead = maxRead

  def checkProductionMetaData(self, metaDict):
    """Make sure NumberOfEvents is set to the number of events after splitting.

    :param dict metaDict: production job metadata dictionary, will be updated
    :returns: S_OK, S_ERROR
    """

    if 'NumberOfEvents' in metaDict:
      metaDict['NumberOfEvents'] = self.numberOfEventsPerFile
    return super(StdHepSplit, self).checkProductionMetaData(metaDict)

  def _applicationModule(self):
    m1 = self._createModuleDefinition()
    m1.addParameter(Parameter("debug", False, "bool", "", "", False, False, "debug mode"))
    m1.addParameter(Parameter("maxRead", 0, "int", "", "", False, False, "max events to read"))
    m1.addParameter(Parameter("nbEventsPerSlice", 0, "int", "", "", False, False,
                                "Number of events per output file"))
    return m1

  def _applicationModuleValues(self, moduleinstance):
    moduleinstance.setValue('debug', self.debug)
    moduleinstance.setValue('nbEventsPerSlice', self.numberOfEventsPerFile)
    moduleinstance.setValue('maxRead', self.maxRead)

  def _userjobmodules(self, stepdefinition):
    res1 = self._setApplicationModuleAndParameters(stepdefinition)
    res2 = self._setUserJobFinalization(stepdefinition)
    if not res1["OK"] or not res2["OK"]:
      return S_ERROR('userjobmodules failed')
    return S_OK()

  def _prodjobmodules(self, stepdefinition):
    res1 = self._setApplicationModuleAndParameters(stepdefinition)
    res2 = self._setOutputComputeDataList(stepdefinition)
    if not res1["OK"] or not res2["OK"]:
      return S_ERROR('prodjobmodules failed')
    return S_OK()

  def _checkConsistency(self, job=None):
    """Checks that all needed parameters are set."""

    # steal the datatype and detector type from the job (for production):
    if hasattr(self._job, "datatype"):
      self.datatype = self._job.datatype

    # This is needed for metadata registration
    self.numberOfEvents = self.numberOfEventsPerFile

    if not self.outputFile and self._jobtype == 'User':
      LOG.notice('No output file name specified.')

    if self._jobtype != 'User':
      self._listofoutput.append({"outputFile": "@{OutputFile}", "outputPath": "@{OutputPath}",
                                 "outputDataSE": '@{OutputSE}'})
      self.prodparameters['nb_events_per_file'] = self.numberOfEventsPerFile

    return S_OK()

  def _checkWorkflowConsistency(self):
    return self._checkRequiredApp()

  def _resolveLinkedStepParameters(self, stepinstance):
    if isinstance(self._linkedidx, six.integer_types):
      self._inputappstep = self._jobsteps[self._linkedidx]
    if self._inputappstep:
      stepinstance.setLink("InputFile", self._inputappstep.getType(), "OutputFile")
    return S_OK()



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Bhlumi Application to run applications based on Bhlumi.

.. versionadded:: v34r0

Usage:

>>> bhlumi = Bhlumi()
>>> bhlumi.setVersion('key4hep-latest')
>>> bhlumi.setRandomSeed(12340)
>>> bhlumi.setEnergy(91.2)
>>> bhlumi.setNumberOfEvents(100)
>>> bhlumi.setOutputFile('output.lhe')

"""

from __future__ import absolute_import
import types
import os

from ILCDIRAC.Interfaces.API.NewInterface.LCApplication import LCApplication
from ILCDIRAC.Core.Utilities.InstalledFiles import Exists
from DIRAC import S_OK, S_ERROR, gLogger
from DIRAC.Core.Workflow.Parameter import Parameter
from DIRAC.ConfigurationSystem.Client.Helpers.Operations import Operations
import six

LOG = gLogger.getSubLogger(__name__)
__RCSID__ = "$Id$"

class Bhlumi(LCApplication):
  """Bhlumi Application Class."""
  def __init__(self, paramdict=None):
    self.randomSeed = -1
    self.debugLevel = 0
    self.bhlumiConfigFile = ''
    super(Bhlumi, self).__init__(paramdict)

    # Those 5 need to come after default constructor
    self._modulename = 'BhlumiAnalysis'
    self._moduledescription = 'Module to run Bhlumi'
    self.appname = 'bhlumi'
    self.datatype = 'GEN'
    self._ops = Operations()
    self._extension = 'lhe'

  def setConfigFile(self, bhlumiConfigFilePath):
    """Set the Bhlumi options to be used.

    :param str bhlumiConfigFilePath: Path to the Bhlumi input file.
    """
    self._checkArgs({'bhlumiConfigFilePath': (str,)})

    # Chech if file exist
    if not os.path.isfile(bhlumiConfigFilePath):
      return self._reportError('Bhlumi config file does not exist!')

    # Read file
    self.bhlumiConfigFile = open(bhlumiConfigFilePath).read()

    return None
  
  def setRandomSeed(self, randomSeed):
    """Define random seed to use. Default is the jobID.

    :param int randomSeed: Seed to use during simulation.
    """
    self._checkArgs({'randomSeed': int})
    self.randomSeed = randomSeed
    return S_OK()

  def setArguments(self, args):
    """Define the arguments of the script.

    Alternative to :func:`Bhlumi.setExtraCLIArguments`.

    :param str args: Arguments to pass to the command call
    """
    self._checkArgs({'args': (str,)})
    self.extraCLIArguments = args
    return S_OK()
  
  def setBhlumiConfigFile(self, bhlumiConfigFilePath):
    """Set the bhlumi configuration file.

    :param str bhlumiConfigFilePath: Name of the Bhlumi configuration file path.
    """
    self._checkArgs({'bhlumiConfigFilePath': (str,)})
    if not os.path.isfile(bhlumiConfigFilePath):
      return self._reportError('Bhlumi configuration file does not exist!')
    self.bhlumiConfigFileContent = open(bhlumiConfigFilePath).read()
    return S_OK()
  
  def _userjobmodules(self, stepdefinition):
    res1 = self._setApplicationModuleAndParameters(stepdefinition)
    res2 = self._setUserJobFinalization(stepdefinition)
    if not res1["OK"] or not res2["OK"]:
      return S_ERROR('userjobmodules failed')
    return S_OK()

  def _prodjobmodules(self, stepdefinition):
    res1 = self._setApplicationModuleAndParameters(stepdefinition)
    res2 = self._setOutputComputeDataList(stepdefinition)
    if not res1["OK"] or not res2["OK"]:
      return S_ERROR('prodjobmodules failed')
    return S_OK()
  
  def _checkConsistency(self, job=None):
    """Check consistency of the Bhlumi application, this is called from the `Job` instance.

    :param job: The instance of the job
    :type job: ~ILCDIRAC.Interfaces.API.NewInterface.Job.Job
    :returns: S_OK/S_ERROR
    """

    if not self.version:
      return S_ERROR('No version found!')

    if self.bhlumiConfigFile:
      return S_OK()

    if self._jobtype != 'User':
      self._listofoutput.append({"outputFile": "@{OutputFile}", "outputPath": "@{OutputPath}",
                                 "outputDataSE": '@{OutputSE}'})
      return S_OK()

    if not self.energy and not self.numberOfEvents and not self.outputFile:
      return S_ERROR('No config file set!')
    if not self.energy:
      return S_ERROR('No energy set!')
    if not self.numberOfEvents:
      return S_ERROR('No number of events set!')
    if not self.outputFile:
      return S_ERROR('No output file set!')
    if self.randomSeed < -1:
      return S_ERROR('Random Seed has to be equal or greater than -1')
    return S_OK()

  
  def _applicationModule(self):
    md1 = self._createModuleDefinition()
    md1.addParameter(Parameter("randomSeed", 0, "int", "", "", False, False, "Random seed for the generator"))
    md1.addParameter(Parameter("bhlumiConfigFile", '', "string", "", "", False, False, "Bhlumi steering options"))
    md1.addParameter(Parameter("debug", False, "bool", "", "", False, False, "debug mode"))
    return md1

  def _applicationModuleValues(self, moduleinstance):
    moduleinstance.setValue("bhlumiConfigFile", self.bhlumiConfigFile)
    moduleinstance.setValue("randomSeed", self.randomSeed)
    moduleinstance.setValue("debug", self.debug)

  def _checkWorkflowConsistency(self):
    return self._checkRequiredApp()



#!/usr/local/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test DelphesApp module."""

from __future__ import print_function
from __future__ import absolute_import
import linecache
import unittest
from mock import patch, MagicMock as Mock
from mock import mock_open
from mock import mock as mock_module
from parameterized import parameterized
import six
import os, shutil, tempfile

from ILCDIRAC.Core.Utilities.CombinedSoftwareInstallation import getEnvironmentScript
from DIRAC import gLogger, S_OK, S_ERROR
from ILCDIRAC.Interfaces.API.NewInterface.Applications import DelphesApp
from Tests.Utilities.GeneralUtils import assertEqualsImproved, assertDiracFailsWith, \
    assertDiracSucceeds

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Interfaces.API.NewInterface.Applications.DelphesApp'
BUILTIN_NAME = 'builtins' if six.PY3 else '__builtin__'

gLogger.setLevel("DEBUG")
gLogger.showHeaders(True)

# pylint: disable=protected-access
def cleanup(tempdir):
  """Remove files after run."""
  try:
    shutil.rmtree(tempdir)
  except OSError:
    pass

class DelphesAppTestCase(unittest.TestCase):
  """Base class for the DelphesApp test cases."""

  @classmethod
  def setUpClass(cls):
    """Load the Application file into the linecache to prevent exceptions when mocking the builtin open."""
    from ILCDIRAC.Interfaces.API.NewInterface import Application
    for fName in [Application.__file__, mock_module.__file__]:
      if fName.endswith(('.pyc', '.pyo')):
        fName = fName[:-1]
      linecache.getlines(fName)

  @classmethod
  def tearDownClass(cls):
    """Remove all entries from linecache because we mock builtin open."""
    linecache.clearcache()

  def setUp(self):
    """set up the objects."""
    self.delphes = DelphesApp({})
    self.delphes._jobtype = 'User'
    self.delphes._ops = Mock(name='OpsMock')
    self.curdir = os.getcwd()
    self.tempdir = tempfile.mkdtemp("", dir="./")
    os.chdir(self.tempdir)

  def tearDown(self):
    os.chdir(self.curdir)
    cleanup(self.tempdir)

  def test_setrandomseed(self):
    self.assertFalse(self.delphes._errorDict)
    self.delphes.setRandomSeed(89421)
    self.assertFalse(self.delphes._errorDict)
    assertEqualsImproved(self.delphes.randomSeed, 89421, self)

  def test_setrandomseed_fails(self):
    self.assertFalse(self.delphes._errorDict)
    self.delphes.setRandomSeed(['abc'])
    self.assertIn('_checkArgs', self.delphes._errorDict)

  def test_setexecutable(self):
    self.assertFalse(self.delphes._errorDict)
    self.delphes.setExecutableName("executable")
    self.assertFalse(self.delphes._errorDict)
    assertEqualsImproved(self.delphes.executableName, "executable", self)

  def test_setexecutable_fails(self):
    self.assertFalse(self.delphes._errorDict)
    self.delphes.setExecutableName(None)
    self.assertIn('_checkArgs', self.delphes._errorDict)

  def test_setdetectorcard(self):
    self.assertFalse(self.delphes._errorDict)
    self.delphes.setDetectorCard("detectorcard")
    self.assertFalse(self.delphes._errorDict)
    assertEqualsImproved(self.delphes.detectorCard, "detectorcard", self)
    self.assertFalse(self.delphes.inputSB)
    self.delphes.setDetectorCard("lfn:/mydir/detectorcard")
    assertEqualsImproved(self.delphes.inputSB, ['lfn:/mydir/detectorcard'], self)

  def test_setdetectorcard_fails(self):
    self.assertFalse(self.delphes._errorDict)
    self.delphes.setDetectorCard(1)
    self.assertIn('_checkArgs', self.delphes._errorDict)

  def test_setoutputcard(self):
    self.assertFalse(self.delphes._errorDict)
    self.delphes.setOutputCard("outputcard")
    self.assertFalse(self.delphes._errorDict)
    assertEqualsImproved(self.delphes.outputCard, "outputcard", self)
    self.assertFalse(self.delphes.inputSB)
    self.delphes.setOutputCard("lfn:/mydir/outputcard")
    assertEqualsImproved(self.delphes.inputSB, ['lfn:/mydir/outputcard'], self)

  def test_setoutputcard_fails(self):
    self.assertFalse(self.delphes._errorDict)
    self.delphes.setOutputCard(1)
    self.assertIn('_checkArgs', self.delphes._errorDict)

  def test_setPythia8Card(self):
    self.assertFalse(self.delphes._errorDict)
    with patch("builtins.open", mock_open(read_data='content')), \
         patch('os.path.isfile', new=Mock(return_value=True)):
      res = self.delphes.setPythia8Card("pythia8card.cmd")
      assertDiracSucceeds(res, self)
      assertEqualsImproved(self.delphes.pythia8CardContent, 'content', self)

  def test_setPythiaLHEreader(self):
    self.assertFalse(self.delphes._errorDict)
    res = self.delphes.setPythia8Card("Pythia_LHEinput.cmd")
    assertDiracSucceeds(res, self)
    self.assertIn('! 2) Settings related to output in init(), next() and stat().', self.delphes.pythia8CardContent)

  def test_setPythia8CardEOS(self):
    self.assertFalse(self.delphes._errorDict)
    with patch("builtins.open", mock_open(read_data='content')), \
         patch("os.path.isfile", new=Mock(side_effect=[False, True])), \
         patch.object(self.delphes._ops, 'getValue', return_value=['firstlocation']):
      res = self.delphes.setPythia8Card("p8_ee_tt_ecm365.cmd")
      assertDiracSucceeds(res, self)
      assertEqualsImproved(self.delphes.pythia8CardContent, 'content', self)

  def test_setPythia8Card_fails_file(self):
    self.assertFalse(self.delphes._errorDict)
    res = self.delphes.setPythia8Card("content")
    assertDiracFailsWith(res, 'Pythia8 configuration file does not exist!', self)

  def test_setPythia8Card_fails_argument(self):
    self.assertFalse(self.delphes._errorDict)
    res = self.delphes.setPythia8Card(1)
    self.assertIn('_checkArgs', self.delphes._errorDict)

  def test_setPythia8CardEOS_fails(self):
    self.assertFalse(self.delphes._errorDict)
    with patch("builtins.open", mock_open(read_data='content')), \
         patch("os.path.isfile", new=Mock(side_effect=[False, False])), \
         patch.object(self.delphes._ops, 'getValue', return_value=['firstlocation']):
      res = self.delphes.setPythia8Card("p8_ee_tt_ecm365.cmd")
      assertDiracFailsWith(res, 'Pythia8 configuration file does not exist!', self)

  def test_setarguments(self):
    self.assertFalse(self.delphes._errorDict)
    self.delphes.setArguments("arguments")
    self.assertFalse(self.delphes._errorDict)
    assertEqualsImproved(self.delphes.extraCLIArguments, "arguments", self)
    assertDiracSucceeds(self.delphes.setArguments("arguments"), self)

  def test_setarguments_fails(self):
    self.assertFalse(self.delphes._errorDict)
    self.delphes.setArguments(None)
    self.assertIn('_checkArgs', self.delphes._errorDict)

#--------------------------------------------------------------------------

  def test_resolvelinkedparams(self):
    step_mock = Mock()
    input_mock = Mock()
    input_mock.getType.return_value = {'abc': False}
    self.delphes._linkedidx = 3
    self.delphes._jobsteps = [None, None, None, input_mock]
    assertDiracSucceeds(self.delphes._resolveLinkedStepParameters(step_mock), self)
    step_mock.setLink.assert_called_once_with('InputFile', {'abc': False}, 'OutputFile')

  def test_resolvelinkedparams_noinputstep(self):
    self.delphes._linkedidx = None
    self.delphes._inputappstep = []
    assertDiracSucceeds(self.delphes._resolveLinkedStepParameters(None), self)

  def test_checkworkflow_app_missing(self):
    self.delphes._inputapp = ['some_depdency', 'unavailable_dependency_fail_on_this']
    self.delphes._jobapps = ['myjobapp_1', 'some_dependency']
    assertDiracFailsWith(self.delphes._checkWorkflowConsistency(), 'job order not correct', self)

  def test_checkworkflow_empty(self):
    self.delphes._inputapp = []
    self.delphes._jobapps = []
    assertDiracSucceeds(self.delphes._checkWorkflowConsistency(), self)

  def test_checkworkflow_success(self):
    self.delphes._inputapp = ['some_dependency', 'other_dependencies', 'many_more']
    self.delphes._jobapps = ['ignore_me', 'many_more', 'some_dependency', 'other_dependencies']
    assertDiracSucceeds(self.delphes._checkWorkflowConsistency(), self)

  def test_userjobmodules(self):
    module_mock = Mock()
    assertDiracSucceeds(self.delphes._userjobmodules(module_mock), self)

  def test_prodjobmodules(self):
    module_mock = Mock()
    assertDiracSucceeds(self.delphes._prodjobmodules(module_mock), self)

  def test_userjobmodules_fails(self):
    with patch('%s._setUserJobFinalization' % MODULE_NAME, new=Mock(return_value=S_OK('something'))),\
            patch('%s._setApplicationModuleAndParameters' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_test_err'))):
      assertDiracFailsWith(self.delphes._userjobmodules(None),
                           'userjobmodules failed', self)

  def test_prodjobmodules_fails(self):
    with patch('%s._setApplicationModuleAndParameters' % MODULE_NAME, new=Mock(return_value=S_OK('something'))), \
            patch('%s._setOutputComputeDataList' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_other_test_err'))):
      assertDiracFailsWith(self.delphes._prodjobmodules(None),
                           'prodjobmodules failed', self)

#----------------------------------------------------------------------------
  def test_checkconsistency_noversion(self):
    self.delphes.version = None
    assertDiracFailsWith(self.delphes._checkConsistency(Mock()), 'no version found', self)

  def test_checkconsistency_badexecutable(self):
    self.delphes.version = '134'
    self.delphes.executableName = None
    assertDiracFailsWith(self.delphes._checkConsistency(Mock()), 'Executable not supported. Supported executables: DelphesPythia8_EDM4HEP, DelphesSTDHEP_EDM4HEP, DelphesROOT_EDM4HEP', self)

  def test_checkconsistency_badseed(self):
    self.delphes.version = '134'
    self.delphes.executableName = 'DelphesPythia8_EDM4HEP'
    self.delphes.randomSeed = -2
    assertDiracFailsWith(self.delphes._checkConsistency(Mock()), 'Random Seed has to be equal or greater than -1', self)

  def test_checkconsistency_nodetectorcard(self):
    self.delphes.version = '134'
    self.delphes.executableName = 'DelphesPythia8_EDM4HEP'
    self.delphes.randomSeed = -1
    assertDiracFailsWith(self.delphes._checkConsistency(Mock()), 'Missing detector config-file.', self)

  def test_checkconsistency_nooutputcard(self):
    self.delphes.version = '134'
    self.delphes.executableName = 'DelphesPythia8_EDM4HEP'
    self.delphes.randomSeed = -1
    self.delphes.detectorCard = 'card_IDEA.tcl'
    assertDiracFailsWith(self.delphes._checkConsistency(Mock()), 'Missing output-config-file.', self)

  def test_checkconsistency_nopythia8card(self):
    self.delphes.version = '134'
    self.delphes.executableName = 'DelphesPythia8_EDM4HEP'
    self.delphes.randomSeed = -1
    self.delphes.detectorCard = 'card_IDEA.tcl'
    self.delphes.outputCard = 'edm4hep_IDEA.tcl'
    assertDiracFailsWith(self.delphes._checkConsistency(Mock()), 'Missing Pythia 8 Card. The execution of DelphesPythia8_EDM4HEP would not succeed', self)

  def test_checkconsistency_baddetectorcard(self):
    self.delphes.version = '134'
    self.delphes.executableName = 'DelphesPythia8_EDM4HEP'
    self.delphes.randomSeed = -1
    self.delphes.detectorCard = 'badname'
    assertDiracFailsWith(self.delphes._checkConsistency(Mock()), 'Wrong name for the detector config file. Hint: they all end in ".tcl"', self)

  def test_checkconsistency_badoutputcard(self):
    self.delphes.version = '134'
    self.delphes.executableName = 'DelphesPythia8_EDM4HEP'
    self.delphes.randomSeed = -1
    self.delphes.detectorCard = 'card_IDEA.tcl'
    self.delphes.outputCard = 'badname'
    assertDiracFailsWith(self.delphes._checkConsistency(Mock()), 'Wrong name for the output config file. Hint: they all end in ".tcl"', self)

  def test_checkconsistency_detectorcardnotavailable(self):
    self.delphes.version = '134'
    self.delphes.executableName = 'DelphesPythia8_EDM4HEP'
    self.delphes.randomSeed = -1
    self.delphes.detectorCard = 'badname.tcl'
    res = self.delphes._checkConsistency(Mock())
    assertDiracFailsWith(res, 'badname.tcl is not available locally nor in the software installation', self)

  def test_checkconsistency_outputcardnotavailable(self):
    self.delphes.version = '134'
    self.delphes.executableName = 'DelphesPythia8_EDM4HEP'
    self.delphes.randomSeed = -1
    self.delphes.detectorCard = 'card_IDEA.tcl'
    self.delphes.outputCard = 'badname.tcl'
    res = self.delphes._checkConsistency(Mock())
    assertDiracFailsWith(res, 'badname.tcl is not available locally nor in the software installation', self)

  def test_checkconsistency_badpythia8card(self):
    self.delphes.version = '134'
    self.delphes.executableName = 'DelphesPythia8_EDM4HEP'
    self.delphes.randomSeed = -1
    self.delphes.detectorCard = 'card_IDEA.tcl'
    self.delphes.outputCard = 'edm4hep_IDEA.tcl'
    self.delphes.pythia8CardContent = 'poorcontent'
    res = self.delphes._checkConsistency(Mock())
    assertDiracFailsWith(res, 'Pythia card with unusual content: neither `Beams:LHEF` nor `Beams:eCM` fields present. Please check the content of your card.', self)

  def test_checkconsistency(self):
    self.delphes.version = '134'
    self.delphes.executableName = 'DelphesPythia8_EDM4HEP'
    self.delphes.detectorCard = 'card_IDEA.tcl'
    self.delphes.outputCard = 'edm4hep_IDEA.tcl'
    self.delphes.pythia8CardContent = 'Beams:LHEF\nMain:numberOfEvents'
    self.delphes.outputFile = 'myoutput.file'
    self.delphes._jobtype = 'User'
    assertDiracSucceeds(self.delphes._checkConsistency(Mock()), self)
    self.assertNotIn({'outputFile': '@{OutputFile}', 'outputPath': '@{OutputPath}',
                        'outputDataSE': '@{OutputSE}'}, self.delphes._listofoutput)
    self.assertNotIn('nbevts', self.delphes.prodparameters)
    self.assertNotIn('Process', self.delphes.prodparameters)

  def test_checkconsistency_longfilenames(self):
    self.delphes.version = '134'
    self.delphes.executableName = 'DelphesPythia8_EDM4HEP'
    self.delphes.detectorCard = 'lfn:card_IDEA.tcl'
    self.delphes.outputCard = 'lfn:edm4hep_IDEA.tcl'
    self.delphes.pythia8CardContent = 'Beams:LHEF\nMain:numberOfEvents'
    self.delphes.outputFile = 'myoutput.file'
    self.delphes._jobtype = 'User'
    assertDiracSucceeds(self.delphes._checkConsistency(Mock()), self)
    self.assertNotIn({'outputFile': '@{OutputFile}', 'outputPath': '@{OutputPath}',
                        'outputDataSE': '@{OutputSE}'}, self.delphes._listofoutput)
    self.assertNotIn('nbevts', self.delphes.prodparameters)
    self.assertNotIn('Process', self.delphes.prodparameters)

  def test_checkconsistency_notuserjobtype(self):
    self.delphes.version = '134'
    self.delphes.executableName = 'DelphesSTDHEP_EDM4HEP'
    self.delphes.detectorCard = 'card_IDEA.tcl'
    self.delphes.outputCard = 'edm4hep_IDEA.tcl'
    self.delphes.outputFile = 'myoutput.file'
    self.delphes._jobtype = 'NotaUser'
    assertDiracSucceeds(self.delphes._checkConsistency(Mock()), self)
    self.assertIn({'outputFile': '@{OutputFile}', 'outputPath': '@{OutputPath}',
                        'outputDataSE': '@{OutputSE}'}, self.delphes._listofoutput)
    self.assertNotIn('nbevts', self.delphes.prodparameters)
    self.assertNotIn('Process', self.delphes.prodparameters)

#----------------------------------------------------------------------------

def runTests():
  """Runs our tests."""
  suite = unittest.defaultTestLoader.loadTestsFromTestCase(DelphesAppTestCase)
  testResult = unittest.TextTestRunner(verbosity=2).run(suite)
  print(testResult)


if __name__ == '__main__':
  runTests()
